// Add new constant to support read-only mode for test purposes
const EDIT_MODE_TP_RO  = 0x02;
const EDIT_MODE_TPROC_RO  = 0x04;

// Command function: Add new test purpose as requirement's child or other testpurpose's sibling
function addTestPurpose(uuid) {
	if ((readOnlyMode & EDIT_MODE_TP_RO) != 0)
		return;
	// Get the wrapping block of the target requirement (if it is a requirement)
	// or of the target test purpose's parent requirement (if it is a test purpose)
	var req_elem = $(this).closest('blockquote.indent');
	// Create the child test purpose if there are no children requirements
	if (!req_elem.hasClass('haschildreq')) {
		addNode(extractUUID(req_elem[0].id), null, 'TestPurpose');
	}
}

// Command function: Add new test step as test procedure's child or other test step's sibling
//function addTestStep( uuid ) 
//{
//    if( (readOnlyMode & EDIT_MODE_TPROC_RO) != 0 ) return;
//    var parent = getStartNode();
//    var lastChildUUID = getLastChildUUID( parent );
//    createTestStep( parent, lastChildUUID );
//}

function addTestStepBefore( uuid ) 
{
    if( (readOnlyMode & EDIT_MODE_TPROC_RO) != 0 ) return;
    createTestStep( getStartNode(), getStartNode() );
}

function addTestStepAfter( uuid ) 
{
    if( (readOnlyMode & EDIT_MODE_TPROC_RO) != 0 ) return;
    createTestStep( getStartNode(), uuid );
}

function addCopyTestStepAfter( uuid ) 
{
	if( (readOnlyMode & EDIT_MODE_TPROC_RO) != 0 ) return;
	copyNode(uuid, getStartNode(), false,uuid);
    //createTestStep( getStartNode(), uuid );
}

function testStepUpdate( uuid ){
	var elems = $('.linked');
		for (var i = 0; i < elems.length; i++){
			if(extractUUID(elems[i].id) == uuid)
				elems[i].innerHTML = getUFName(uuid);
		}
}

// Command function: Switch test step type
function changeTestStepType(uuid) {
    var tstype = $('#tstype-' + uuid);
    var currentTSType = tstype.text();
    var newTSType;
    if( currentTSType == 'act' )    newTSType = 'verify'; else
    if( currentTSType == 'verify' ) newTSType = 'ensure'; else
    if( currentTSType == 'ensure' ) newTSType = 'act';
    
    var nodeType = getNodeType($(this));

    tstype.text( newTSType );
    
    changeNodeAttr(uuid, "_tstype", newTSType );
    updateTree(uuid);
}

function openTestStepLink(uuid){
	var nodeData = $.parseJSON(getNodeByUUID(uuid));
	var tp = nodeData.attributes._tslink.value;
	openEditorForNode(tp);
}

function unlinkTestPurpose(uuid){
	var nodeData = $.parseJSON(getNodeByUUID(uuid));
	changeNodeAttr(uuid,'_tslink',null);
	updateTree(uuid);
}

function linkTestPurpose(uuid)
{
	var tp = addTestPurposeToTestStep(uuid);
	if(tp != null)
		updateTree(uuid);
}

function specifyInitialConditions(uuid)
{
//    saveEditor();
//    $(".tprocinitcondwrapper").css("display","block");
    putEditor($(".tprocinitcond").get(0),false);
}

$(function () {
    var startNode = $.parseJSON( getNodeByUUID( getStartNode() ) );
    var startNodeType = startNode.attributes._type.value;
    var testProcedureMode = (startNodeType == "TestProcedure");
    
	// Register new hotkey for adding test purposes
	hotkeyManager.registerHotkeys({
		'Ins': { handlerGlobal: testProcedureMode ? addTestStepAfter : addTestPurpose, handlerCKEditor: null }
	});

	// Implement Save actions for test purpose specific fields
	saveAction['tpid'] = function (uuid, new_data) {
		new_data = new_data.unescapeHtml();
		if(!checkName(uuid, new_data))
			new_data = "";
		changeNodeId(uuid, new_data);
	};
	saveAction['tpdesc'] = function (uuid, new_data) {
		// Split to Description and Expected Results
		var desc;
		var exp_res;
		var pos = new_data.indexOf('-- Expected results:');
		if (pos >= 0) {
			desc = new_data.substr(0, pos).replace(/<br\s*\/>\s*$/, '');
			exp_res = new_data.substr(pos + 20).replace(/^\s*<br\s*\/>/, '');
			// Collect all tags that remain opened at the "Expected results" delimiter
			var tag_stack = [];
			pos = 0;
			while ((pos = desc.indexOf('<', pos)) != -1) {
				// Tag started...
				var pos_end = desc.indexOf('>', pos);
				if (pos_end == -1)
					break;   // Corrupted XHTML, stop parsing
				if (desc[pos + 1] == '/') {
					// This is a closing tag - remove from the stack
					var tag_name = desc.substring(pos + 2, pos_end);
					var last_tag = tag_stack.pop();
					if (tag_name != last_tag.replace(/ .*$/, '')) {
						tag_stack.push(last_tag);
					}
				}
				else {
					if (desc[pos_end - 1] != '/') {
						// This is an opening and not self-closing tag - push to the stack
						tag_stack.push(desc.substring(pos + 1, pos_end));
					}
				}
				pos = pos_end;
			}
			// Close all opened tags in the end of first part (Description)
			// and duplicate them at the beginning of the second part (Expected results)
			var exp_res_prefix = '';
			var last_tag;
			while (last_tag = tag_stack.pop()) {
				desc += '</' + last_tag.replace(/ .*$/, '') + '>';
				exp_res_prefix = '<' + last_tag + '>' + exp_res_prefix;
			}
			exp_res = exp_res_prefix + exp_res;
		}
		else {
			desc = new_data;
			exp_res = '';
		}
		changeNodeAttr(uuid, '_description', desc);
		changeNodeAttr(uuid, '_expectedResults', exp_res);
	};
    saveAction['tprocid'] = function (uuid, new_data) {
    	new_data = new_data.unescapeHtml();
		if(!checkName(uuid, new_data))
			new_data = "";
        changeNodeId(uuid, new_data);
    };
    saveAction['tprocdesc'] = function (uuid, new_data) {
        changeNodeAttr(uuid, '_description', new_data);
    }
    saveAction['tprocinitcond'] = function (uuid, new_data) {
        changeNodeAttr(uuid, '_initcond', new_data);
    }
    saveAction['tsdesc'] = function (uuid, new_data) {
        changeNodeAttr(uuid, '_description', new_data);
    }

	// Add read-only class binding
	readOnlyClasses[EDIT_MODE_TP_RO] = 'testpurpose';
    readOnlyClasses[EDIT_MODE_TPROC_RO] = 'testprocedure';
    readOnlyClasses[EDIT_MODE_TPROC_RO] = 'teststep';

	// Append command to Requirement nodes for creating test purpose children
	node_commands['Requirement']['addtp'] = {
		menu: true,
		name: 'Add new test purpose',
		image: 'add-tp.png',
		imageAlt: '[+]',
		readOnlyMode: EDIT_MODE_TP_RO,
		handler: addTestPurpose
	};
	// Add drop-down menu to test purpose nodes
	node_commands['TestPurpose'] = {
		'deltp': {
			menu: true,
			name: 'Delete test purpose',
			image: 'delete-node.png',
			imageAlt: '[&minus;]',
			readOnlyMode: EDIT_MODE_TP_RO,
			handler: deleteNodeConfirm
		},
		'addtp': {
			menu: true,
			name: 'Add new test purpose',
			image: 'add-tp.png',
			imageAlt: '[+]',
			readOnlyMode: EDIT_MODE_TP_RO,
			handler: addTestPurpose
		},
		'addcmt': {
			menu: true,
			name: 'Add new comment',
			image: 'add-cmt.png',
			imageAlt: '[+]',
			readOnlyMode: 0,
			handler: addComment
		},
		'chstatus': {
			menu: false,
			name: 'Change status',
			readOnlyMode: 0,
			handler: changeStatus
		}
	};
    // Add drop-down menu to test procedure nodes
    node_commands['TestProcedure'] = {
        'addts': {
            menu: true,
            name: 'Add new test step',
            image: 'add-ts.png',
            imageAlt: '[+]',
            readOnlyMode: EDIT_MODE_TPROC_RO,
            handler: addTestStepBefore
        },
//        'specifyinitconds': {
//            menu: true,
//            name: 'Specify initial conditions',
//            image: 'specify-initconds.png',
//            imageAlt: '[+]',
//            readOnlyMode: EDIT_MODE_TPROC_RO,
//            handler: specifyInitialConditions
//        },
        'chstatus': {
            menu: false,
            name: 'Change status',
            readOnlyMode: 0,
            handler: changeStatus
        }
    };
    // Add drop-down menu to test procedure steps
    node_commands['TestStep'] = {
        'delts': {
            menu: true,
            name: 'Delete test step',
            image: 'delete-node.png',
            imageAlt: '[&minus;]',
            readOnlyMode: EDIT_MODE_TPROC_RO,
            handler: deleteNodeConfirm
        },
        'addts': {
            menu: true,
            name: 'Add new test step',
            image: 'add-ts.png',
            imageAlt: '[+]',
            readOnlyMode: EDIT_MODE_TPROC_RO,
            handler: addTestStepAfter
        },
		'addtssib': {
			menu: true,
			name: 'Clone test step',
			image: 'add-ts.png',
			imageAlt: '[+]',
			readOnlyMode: EDIT_MODE_TPROC_RO,
			handler: addCopyTestStepAfter
		},
        'linktp': {
            menu: true,
            name: 'Link test purpose',
            image: 'link-tp.png',
            imageAlt: '[+]',
            readOnlyMode: EDIT_MODE_TPROC_RO,
            handler: linkTestPurpose
        },
        'unlinktp': {
            menu: true,
            name: 'Unlink test purpose',
            image: 'unlink-tp.png',
            imageAlt: '[+]',
            readOnlyMode: EDIT_MODE_TPROC_RO,
            handler: unlinkTestPurpose
        },
        'openlink': {
            menu: false,
            name: 'Link test purpose',
            readOnlyMode: 0,
            handler: openTestStepLink
        },
        'chtstype': {
            menu: false,
            name: 'Change test step type',
            readOnlyMode: 0,
            handler: changeTestStepType
        }
    };

	// Fill in list of statuses
	node_statuses['TestPurpose'] = {
		'in process': {
			icon: 'tp-status-incomplete.png',
			className: 'status-normal'
		},
		'complete': {
			icon: 'tp-status-complete.png',
			className: 'status-complete'
		},
		'verified': {
			icon: 'tp-status-verified.png',
			className: 'status-verified'
		}
	};
    node_statuses['TestProcedure'] = {
        'in process': {
            icon: 'tproc-status-incomplete.png',
            className: 'status-normal'
        },
        'complete': {
            icon: 'tproc-status-complete.png',
            className: 'status-complete'
        },
        'verified': {
            icon: 'tproc-status-verified.png',
            className: 'status-verified'
        }
    };
    node_statuses['TestStep'] = {
        'in process': {
            icon: 'ts-status-incomplete.png',
            className: 'status-normal'
        },
        'complete': {
            icon: 'ts-status-complete.png',
            className: 'status-complete'
        },
        'verified': {
            icon: 'ts-status-verified.png',
            className: 'status-verified'
        }
    };

	// Insert TestPurpose node type into the order list at the last but one position (just before Requirement)
	childrenNodesOrder.splice(-1, 0, 'TestPurpose','TestProcedure','TestStep');
	// Add test purpose node constructors
	constructNodeWrapperProcessor['TestPurpose'] = function (uuid) {
		return $('<div class="testpurposes"><div class="testpurpose-title">Test purposes:</div></div>');
	};

    constructNodeProcessor['TestPurpose'] = function (nodeData, uuid) {
		// Fill in default values for missing attributes
		if (!nodeData.attributes._description || !nodeData.attributes._description.value)
			nodeData.attributes._description = { 'value': '&nbsp;' };
		if (!nodeData.attributes._status || !nodeData.attributes._status.value)
			nodeData.attributes._status = { 'value': 'in process' };
		// Combine description and expected results together
		var expectedResults = '';
		if (nodeData.attributes._expectedResults && nodeData.attributes._expectedResults.value)
			expectedResults = '<br />-- Expected results:<br />' + nodeData.attributes._expectedResults.value;
		// Construct HTML block
		var editable_prefix = (((readOnlyMode & EDIT_MODE_TP_RO) == 0) ? '' : 'non');
		var newNode = $(
			'<div class="tpwrapper nodewrapper" id="block-' + uuid + '">' +
			  '<div style="height: 1px; width: 100%; border-top: solid 1px gray;"></div>' +
			  '<div class="testpurpose supported-node" id="node-' + uuid + '">' +
			    '<table cellpadding="0" cellspacing="0" border="0" width="100%" class="tpid_wrap">' +
			      '<tr>' +
			        '<td>' +
			          '<img src="images/' + node_statuses['TestPurpose'][nodeData.attributes._status.value].icon + '" alt="' + nodeData.attributes._status.value + '" title="' + nodeData.attributes._status.value + '" id="status-' + uuid + '" class="imgstatus" onclick="javascript:execNodeCommand(\'TestPurpose\', \'' + uuid + '\', \'chstatus\');" />' +
			        '</td>' +
			        '<td width="100%">' +
			          '<div class="tpid nodetitle ' + editable_prefix + 'renamable ' + node_statuses['TestPurpose'][nodeData.attributes._status.value].className + '" id="tpid-' + uuid + '">' +
			            nodeData.id +
			          '</div>' +
			        '</td>' +
			        '<td>' +
			          '<div class="tpctrls nodectrls">' +
			            '<div style="text-align: right;">' +
			              '<img src="images/menu.png" alt="[v]" />' +
			            '</div>' +
			            '<div class="menu" style="display: none;">' +
			              constructMenu('TestPurpose', uuid) +
			            '</div>' +
			          '</div>' +
			        '</td>' +
			      '</tr>' +
			    '</table>' +
			    '<div class="tpdesc nodedesc ' + editable_prefix + 'editable" id="tpdesc-' + uuid + '">' +
			      nodeData.attributes._description.value + expectedResults +
			    '</div>' +
			  '</div>' +
			'</div>'
		);

		// Enumerate all children objects and recursively construct the rest of the tree
		var childrenNodes = {};
		for (var i in nodeData.children) {
			// Construct the next child subtree (for requirements) or table (for test purposes)
			var chldNode = constructTree(nodeData.children[i]);
			var chldType = chldNode.type;
			if (!childrenNodes[chldType]) {
				var wrapperConstructor = constructNodeWrapperProcessor[chldType];
				if (wrapperConstructor)
					childrenNodes[chldType] = wrapperConstructor(uuid);
				else
					childrenNodes[chldType] = $('<div />');
			}
			childrenNodes[chldType].append(chldNode);
		}
		// Now append children nodes blocks to the current requirement node
		for (var i = 0; i < childrenNodesOrder.length; ++i) {
			var chldType = childrenNodesOrder[i];
			if (childrenNodes[chldType]) {
				newNode.append(childrenNodes[chldType]);
			}
		}

		return newNode;
	};


    // Add test procedure node constructors
	// NOT USED - always top-level element
    constructNodeWrapperProcessor['TestProcedure'] = function (uuid) {
        return $('<div class="testprocedures"><div class="testprocedure-title">Test procedure:</div></div>');
    };
    
    constructNodeProcessor['TestProcedure'] = function (nodeData, uuid) {
        // Fill in default values for missing attributes
//        if (!nodeData.attributes._description || !nodeData.attributes._description.value)
//            nodeData.attributes._description = { 'value': '&nbsp;' };
        var description;
        if( nodeData.attributes._description && nodeData.attributes._description.value )
            description = nodeData.attributes._description.value;
        else
            description = "&nbsp;";

        var initcond;
        if( nodeData.attributes._initcond && nodeData.attributes._initcond.value )
            initcond = nodeData.attributes._initcond.value;
        else
            initcond = "&nbsp;";
        

        if (!nodeData.attributes._status || !nodeData.attributes._status.value)
            nodeData.attributes._status = { 'value': 'in process' };
        // Construct HTML block
        var editable_prefix = (((readOnlyMode & EDIT_MODE_TPROC_RO) == 0) ? '' : 'non');
        var newNode = $(
            '<div class="tprocwrapper nodewrapper" id="block-' + uuid + '">' +
//              '<div style="height: 1px; width: 100%; border-top: solid 1px gray;"></div>' +
              '<div class="testprocedure supported-node" id="node-' + uuid + '">' +
                '<table cellpadding="0" cellspacing="0" border="0" width="100%" class="tprocid_wrap">' +
                  '<tr>' +
                    '<td>' +
                      '<img src="images/' + node_statuses['TestProcedure'][nodeData.attributes._status.value].icon + '" alt="' + nodeData.attributes._status.value + '" title="' + nodeData.attributes._status.value + '" id="status-' + uuid + '" class="imgstatus" onclick="javascript:execNodeCommand(\'TestProcedure\', \'' + uuid + '\', \'chstatus\');" />' +
                    '</td>' +
                    '<td width="100%">' +
                      '<div class="tprocid nodetitle ' + editable_prefix + 'renamable ' + node_statuses['TestProcedure'][nodeData.attributes._status.value].className + '" id="tprocid-' + uuid + '">' +
                        nodeData.id +
                      '</div>' +
                    '</td>' +
                    '<td>' +
                      '<div class="tprocctrls nodectrls">' +
                        '<div style="text-align: right;">' +
                          '<img src="images/menu.png" alt="[v]" />' +
                        '</div>' +
                        '<div class="menu" style="display: none;">' +
                          constructMenu('TestProcedure', uuid) +
                        '</div>' +
                      '</div>' +
                    '</td>' +
                  '</tr>' +
                '</table>' +
                '<div class="tprocdesc nodedesc ' + editable_prefix + 'editable" id="tprocdesc-' + uuid + '">' +
                  description +
                '</div>' +
//                '<div class="tprocinitcondwrapper" style="display: ' + (initcond == '' || initcond == '&nbsp;' ? 'none' : 'block' ) + ';">' +
                '<div class="tprocinitcondwrapper">' +
                  '<b>Initial conditions: </b>' +
                  '<div class="tprocinitcond ' + editable_prefix + 'editable" id="tprocinitcond-' + uuid + '">' +
                    initcond +
                  '</div>' +
                '</div>' +  
              '</div>' +
            '</div>'
        );
        // Enumerate all children objects and recursively construct the rest of the tree
        var childrenNodes = {};
        for (var i in nodeData.children) {
            // Construct the next child subtree (for requirements) or table (for test purposes)
            var chldNode = constructTree(nodeData.children[i]);
            var chldType = chldNode.type;
            if (!childrenNodes[chldType]) {
                var wrapperConstructor = constructNodeWrapperProcessor[chldType];
                if (wrapperConstructor)
                    childrenNodes[chldType] = wrapperConstructor(uuid);
                else
                    childrenNodes[chldType] = $('<div />');
            }
            childrenNodes[chldType].append(chldNode);
        }
        // Now append children nodes blocks to the current requirement node
        for (var i = 0; i < childrenNodesOrder.length; ++i) {
            var chldType = childrenNodesOrder[i];
            if (childrenNodes[chldType]) {
                newNode.append(childrenNodes[chldType]);
            }
        }

        return newNode;
    };
    

    // Add test procedure node constructors
    constructNodeWrapperProcessor['TestStep'] = function (uuid) {
        return $('<div class="teststeps"></div>');
    };
    
    constructNodeProcessor['TestStep'] = function (nodeData, uuid) {
        // Fill in default values for missing attributes
        if (!nodeData.attributes._tstype || !nodeData.attributes._tstype.value)
            nodeData.attributes._tstype = { 'value': 'act' };
        if (!nodeData.attributes._description || !nodeData.attributes._description.value)
            nodeData.attributes._description = { 'value': '&nbsp;' };
        if (!nodeData.attributes._status || !nodeData.attributes._status.value)
            nodeData.attributes._status = { 'value': 'in process' };
        // Construct HTML block
        var string1 = '<div class="tsdesc nodedesc ' + editable_prefix + 'editable renamable" id="tsdesc-' + uuid + '">' +
                      nodeData.attributes._description.value +
                    '</div>',string2 = "";
        if (nodeData.attributes._tslink != null && 
        		nodeData.attributes._tslink.value != null &&
        		nodeData.attributes._tstype.value == "verify"){
        			string2 = '<tr><td colspan="3">'+string1+'</td></tr>';
        			string1 = '<u><span class="linked" id="linked-' + nodeData.attributes._tslink.value + '" onclick="javascript:execNodeCommand(\'TestStep\', \'' + uuid + '\', \'openlink\');">' + getUFName(nodeData.attributes._tslink.value) + '</span></u>';
        		}
        var editable_prefix = (((readOnlyMode & EDIT_MODE_TPROC_RO) == 0) ? '' : 'non');
        var newNode = $(
            '<div class="tswrapper nodewrapper" id="block-' + uuid + '">' +
//              '<div style="height: 1px; width: 100%; border-top: solid 1px gray;"></div>' +
              '<div class="teststep supported-node dropable" id="node-' + uuid + '">' +
                '<table cellpadding="0" cellspacing="0" border="0" width="100%" class="tsid_wrap">' +
                	'<tr>' +
//                    '<td>' +
//                      '<img src="images/' + node_statuses['TestStep'][nodeData.attributes._status.value].icon + '" alt="' + nodeData.attributes._status.value + '" title="' + nodeData.attributes._status.value + '" id="status-' + uuid + '" class="imgstatus" onclick="javascript:execNodeCommand(\'TestStep\', \'' + uuid + '\', \'chstatus\');" />' +
//                    '</td>' +
//                    '<td width="100%">' +
//                      '<div class="tsid nodetitle ' + editable_prefix + 'renamable ' + node_statuses['TestStep'][nodeData.attributes._status.value].className + '" id="tsid-' + uuid + '">' +
//                        nodeData.id +
//                      '</div>' +
//                    '</td>' +
                    '<td style="vertical-align: '+((string2 == '') ? 'top' : 'center')+';"><div class="tstype" id="tstype-' + uuid + '" onclick="javascript:execNodeCommand(\'TestStep\', \'' + uuid + '\', \'chtstype\');">' + nodeData.attributes._tstype.value + '</div></td>' +
                    '<td width="100%">'+string1+'</td>' +
                    '<td style="vertical-align: top;">' +
                      '<div class="tsctrls nodectrls">' +
                        '<div style="text-align: right;">' +
                          '<img src="images/menu.png" alt="[v]" />' +
                        '</div>' +
                        '<div class="menu" style="display: none;">' +
                          constructMenu('TestStep', uuid) +
                        '</div>' +
                      '</div>' +
                    '</td>' +
                  '</tr>' +
                  string2 +
                '</table>' +
//                '<div class="tsdesc nodedesc ' + editable_prefix + 'editable" id="tsdesc-' + uuid + '">' +
//                  nodeData.attributes._description.value +
//                '</div>' +
              '</div>' +
            '</div>'
        );
        if((nodeData.attributes._tstype.value!="verify"))
        	$(newNode.find('.menu-item-linktp')[0]).remove();
        if(string2 == ""){
        	$(newNode.find('.menu-item-unlinktp')[0]).remove();
        }else{
        	($(newNode.find('.menu-item-linktp-name'))[0]).innerHTML='Link other test purpose';
        }
        // Enumerate all children objects and recursively construct the rest of the tree
        var childrenNodes = {};
        for (var i in nodeData.children) {
            // Construct the next child subtree (for requirements) or table (for test purposes)
            var chldNode = constructTree(nodeData.children[i]);
            var chldType = chldNode.type;
            if (!childrenNodes[chldType]) {
                var wrapperConstructor = constructNodeWrapperProcessor[chldType];
                if (wrapperConstructor)
                    childrenNodes[chldType] = wrapperConstructor(uuid);
                else
                    childrenNodes[chldType] = $('<div />');
            }
            childrenNodes[chldType].append(chldNode);
        }
        // Now append children nodes blocks to the current requirement node
        for (var i = 0; i < childrenNodesOrder.length; ++i) {
            var chldType = childrenNodesOrder[i];
            if (childrenNodes[chldType]) {
                newNode.append(childrenNodes[chldType]);
            }
        }
        //alert(nodeData.attributes._tstype.value);
        return newNode;
    };

	// Add class-to-type mapping for new node type
	nodeClassToType['testpurpose'] = 'TestPurpose';
	nodeClassToType['testprocedure'] = 'TestProcedure';
	nodeClassToType['teststep'] = 'TestStep';

	// Allow paste comments into test purposes
	pasteAllowedMatrix['Comment'].push('TestPurpose');
	// Allow paste test purposes into requirements
	pasteAllowedMatrix['TestPurpose'] = [ 'Requirement' ];
	// Allow pase test step into test procedure
	pasteAllowedMatrix['TestStep'] = ['TestProcedure' ];

	// Add name of the status attribute for the new node type
	statusAttr['TestPurpose'] = '_status';
    statusAttr['TestProcedure'] = '_status';
    statusAttr['TestStep'] = '_status';

	// Customize CKEditor style for editing test purpose ID
	CKEditorStyles['tpid'] = 'body { margin: 1px; font-weight: bold; }';
    CKEditorStyles['tprocid'] = 'body { margin: 1px; font-weight: bold; }';
    CKEditorStyles['tsdesc'] = 'body { margin: 1px; font-weight: bold; }';

	// Add specific dragging init handler
	draggingInitHandler['tpid'] = function (dragging_uuid, req_elems) {
		var req_uuid = extractUUID($('#node-' + dragging_uuid).closest('blockquote')[0].id);
		var underlying_req_idx = req_elems.index($('#node-' + req_uuid));
		if (underlying_req_idx < 0) {
			underlying_req_idx = null;
		}
		// Disallow dragging onto the parent requirement, and onto all non-leaves
		$('#node-' + req_uuid + ', .collapsable > .requirement').addClass('undroppable');
		// Set the disabled look for parent requirement and all non-leaves
		$('#block-' + req_uuid + ', .collapsable > .requirement, .collapsable > .testpurposes, .testpurposes').addClass('disabled');
		for(var i = 0; i < req_elems.length; i++){
			var kuuid = extractUUID(req_elems[i].id);
			if(!checkChild(dragging_uuid, "", kuuid)){
		        $('#node-' + kuuid).addClass('undroppable');
		        $('#node-' + kuuid).addClass('disabled');
			}
		}
		return underlying_req_idx;
	};
	// Add specific dragging init handler
	draggingInitHandler['tsdesc'] = function (dragging_uuid, req_elems) {
		var req_uuid = extractUUID($('#node-' + dragging_uuid).closest('div .nodewrapper')[0].id);
		var underlying_req_idx = req_elems.index($('#node-' + req_uuid));
		if (underlying_req_idx < 0) {
			underlying_req_idx = null;
		}
		return underlying_req_idx;
	};
	
});
