/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.editors.browser;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.eclipse.handlers.location.CommonRemoveLocation;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

final class RemoveLocationFromRequirement extends Action {
    private final ReqMarker reqMarker;

    RemoveLocationFromRequirement(ReqMarker reqMarker) {
        super("Remove from Requirements");
        this.reqMarker = reqMarker;
    }

    public boolean isEnabled() {
        try {
            return reqMarker.getBrowser().isRemovable();
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public void run() {
        String locQId = reqMarker.getDocument().getQualifiedId() + "/"
                + reqMarker.getBrowser().getSelectedLocationId();
        Location loc = (Location) reqMarker.getTreeDB().getNode(locQId);
        Requirement[] req4check = Location.getRequirementsFor(reqMarker.getTreeDB(), locQId);
        
        CommonRemoveLocation.RemoveLocation(loc, null, reqMarker, false);
        
        List<Requirement> reqs = new ArrayList<Requirement>();
        String strList = "";
        for (Requirement req : req4check) {
            if (req.getLocations().size() == 0
                    && req.sizeChildren() == 0) {
                reqs.add(req);
                strList += req.getNameOrId() + "\n";
            }
        }
        if (reqs.size() > 0) {
            if (MessageDialog.openQuestion(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(),
                    "Delete Requirement", (reqs.size() == 1 ? "Requirement\n" + reqs.get(0).getNameOrId() + "\nhas " 
                                                            : "Requirements\n" + strList + "have ") +
                            "no locations, delete?")) {
                reqs.get(0).getTreeDB().startTransaction("Delete Requirement" + (reqs.size() > 1 ? "s" : ""));
                for (Requirement req : reqs) {
                    req.delete();
                }
                reqs.get(0).getTreeDB().commit();
            }
        }
        reqMarker.fireSelectionChange();
    }
}
