package com.unitesk.requality.eclipse.handlers;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateTestPurposeHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        Requirement req = (Requirement) RequalityCNF.getSelectedNode();
        String testPurposeId = getTestPurposeId(req);
        if(!req.canBeParentOf(TestPurpose.TYPE_NAME))
            return null;
        req.getTreeDB().startTransaction("Create Test Purpose");
        TestPurpose tc = (TestPurpose) req.createChild(testPurposeId,
                TestPurpose.TYPE_NAME);
        tc.putAttribute(new Attribute(AttributeType.STRING,
                TestPurpose.ATTR_AUTHOR, System.getProperty("user.name", "")));
        tc.putAttribute(new Attribute(AttributeType.STRING,
                TestPurpose.ATTR_STATUS, "in process"));
        tc.saveAttributes();
        req.getTreeDB().commit();
        Activator.getDefault().setActiveNode(tc, TestPurpose.TYPE_NAME);
        return null;
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node == null) {
            return false;
        }
        if (!node.getType().equals(Requirement.TYPE_NAME)) {
            return false;
        }
        if (node.getQualifiedId().equals(Requirement.getTypeRootQId())) {
            return false;
        }
        if (countNotTestPurposeChildren(node) > 0) {
            return false;
        }
        return true;
    }

    private int countNotTestPurposeChildren(TreeNode node) {
        int count = 0;
        for (int i = 0; i < node.sizeChildren(); i++) {
            if (node.getChild(i).getType().equals(TestPurpose.TYPE_NAME)) {
                continue;
            }
            if (node.getChild(i).getType().equals(Comment.TYPE_NAME)) {
                continue;
            }
            count++;
        }
        return count;
    }

    private String getTestPurposeId(Requirement node) {
        List<TestPurpose> testPurposes = new ArrayList<TestPurpose>();
        for (int i = 0; i < node.sizeChildren(); i++) {
            if (node.getChild(i) instanceof TestPurpose) {
                testPurposes.add((TestPurpose) node.getChild(i));
            }
        }

        if (testPurposes.size() == 0) {
            return "01";
        }

        TreeNode parent = testPurposes.get(0).getParent();
        return RequalityCLI.getNextId(parent, TestPurpose.TYPE_NAME, null);
    }
}
