/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.handlers.requirement;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.RequalityCLI;

public class CreateRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode node = RequalityCNF.getSelectedNode();
        String id = RequalityCLI.getNextId(node, Requirement.TYPE_NAME, null);
        TreeDB db = node.getTreeDB();
        if(!node.canBeParentOf(Requirement.TYPE_NAME))
            return null;
        db.startTransaction("Create Requirement");
        node.createChild(id, Requirement.TYPE_NAME);
        node.saveAttributes();
        db.commit();
        return null;
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node == null) {
            return false;
        }
        if (!node.getType().equals(Requirement.TYPE_NAME)) {
            return false;
        }
        if (countNotRequirementChildren(node) > 0) {
            return false;
        }
        return true;
    }

    private int countNotRequirementChildren(TreeNode node) {
        int count = 0;
        for (int i = 0; i < node.sizeChildren(); i++) {
            if (node.getChild(i).getType().equals(Requirement.TYPE_NAME)) {
                continue;
            }
            if (node.getChild(i).getType().equals(Comment.TYPE_NAME)) {
                continue;
            }
            count++;
        }
        return count;
    }

}
