/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.handlers.requirement;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.handlers.location.CommonRemoveLocation;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class DeleteRequirementHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        TreeNode[] nodes = new TreeNode[1];
        nodes[0] = RequalityCNF.getSelectedNode();
        TreeNode parent = nodes[0].getParent();
        if (nodes.length == 1) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(),
                    "Delete Requirement", "Delete selected "
                            + nodes[0].getType() + "?")) {
                return null;
            }
        } else {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Delete Nodes",
                    "Delete selected nodes")) {
                return null;
            }
        }
        for (TreeNode node : nodes) {
            if (!(node.getParent() instanceof Requirement))
                continue;
            TreeDB db = node.getTreeDB();
            db.startTransaction("Delete Requirement");
            deleteLocations((Requirement)node);
            node.delete();
            db.commit();
        }
        if (parent != null)
            Activator.getDefault().setActiveNode(parent, parent.getType());
        return null;
    }

    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            CommonRemoveLocation.RemoveLocation(loc, req, null, true);
            //TODO: save jsData for undo/redo
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }

    @Override
    public boolean isEnabled() {
        TreeNode node = RequalityCNF.getSelectedNode();
        if (node == null)
            return false;
        if (node.getQualifiedId().equals(Requirement.getTypeRootQId()))
            return false;
        if (!node.getType().equals(Requirement.TYPE_NAME)) {
            return false;
        }
        return super.isEnabled();
    }
}
