/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.navigator.CommonViewer;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.core.IUIStateChangeListener;
import com.unitesk.requality.eclipse.core.TreesTracker;

public class RequalityCNF extends CommonNavigator implements
        ITabbedPropertySheetPageContributor {
    public final static String ID = "com.unitesk.requality.views.RequalityExplorer";
    ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
        }

        @Override
        public void deleted(NodeDesc nodeDesc) {
            RequalityCNF.this.refresh(db.getNode(nodeDesc.getParentUUId()));
        }

        @Override
        public void changed(final TreeNode node) {
            UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                    RequalityCNF.this.refresh(node);
                    return Status.OK_STATUS;
                }
            };
            job.schedule();
            return;
        }

        @Override
        public void created(TreeNode node) {
        }
    };
    private TreeDB db = null;
    final private IUIStateChangeListener uiListener = new IUIStateChangeListener() {
        @Override
        public void newActiveNode(TreeNode node) {
            if (node == null) {
                UIJob job = new UIJob("update selection") {
                    @Override
                    public IStatus runInUIThread(IProgressMonitor monitor) {
                        RequalityCNF.this.getCommonViewer().setSelection(StructuredSelection.EMPTY);
                        return Status.OK_STATUS;
                    }
                };
                job.schedule();
                return;
            }
            Object obj = ((IStructuredSelection) getCommonViewer()
                    .getSelection()).getFirstElement();

            if (obj != null && obj.equals(node))
                return;
            Object parentElement = node.getParent();
            if(db != null)
                db.removeListener(treeChangeListener);
            db = node.getTreeDB();
            db.addListener(treeChangeListener);
            StructuredSelection element = new StructuredSelection(node);
            getCommonViewer().refresh(node.getParent());
            getCommonViewer().expandToLevel(parentElement, 1);
            selectReveal(element);
        }
    };

    final private ISelectionChangedListener treeSelection = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if (event.getSelection() instanceof IStructuredSelection) {
                Object obj = ((IStructuredSelection) event.getSelection())
                        .getFirstElement();
                if (obj instanceof TreeNode) {
                    TreeNode node = (TreeNode) obj;
                    if(db != null)
                        db.removeListener(treeChangeListener);
                    db = node.getTreeDB();
                    db.addListener(treeChangeListener);
                    if (node == null
                            || node.equals(Activator.getDefault()
                                    .getActiveNode(node.getType())))
                        return;
                    Activator.getDefault().setActiveNode(node, node.getType());
                }
            }
        }
    };

    public RequalityCNF() {
        super();
    }

    @Override
    public void dispose() {
        super.dispose();
        getCommonViewer().removePostSelectionChangedListener(treeSelection);
        Activator.getDefault().removeChangeListener(uiListener);
        if(db != null)
            db.removeListener(treeChangeListener);
    }

    @Override
    public void createPartControl(Composite aParent) {
        super.createPartControl(aParent);
        Activator.getDefault().addChangeListener(uiListener);
        getCommonViewer().addPostSelectionChangedListener(treeSelection);
    }

    // This is danger hack
    @Override
    protected CommonViewer createCommonViewerObject(Composite aParent) {
        CommonViewer cv = new CommonViewer(getViewSite().getId(), aParent,
                SWT.H_SCROLL | SWT.V_SCROLL);
        return cv;
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequalityPropertyContributor";
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class adapter) {
        if (adapter == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(this);
        }
        return super.getAdapter(adapter);
    }

    public static synchronized TreeNode getSelectedNode() {

        IWorkbenchWindow activeWorkbenchWindow = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (activeWorkbenchWindow == null)
            return null;
        ISelection selection = activeWorkbenchWindow.getSelectionService()
                .getSelection();

        if (!(selection instanceof IStructuredSelection)) {
            return null;
        }
        IStructuredSelection selected = (IStructuredSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }

        Object obj = selected.getFirstElement();
        if (obj instanceof TreeNode) {
            TreeNode node = (TreeNode) obj;
            return node;
        } else if (obj instanceof IFile) {
            TreeNode node = TreesTracker.getNodeByResource((IFile) obj); 
            return node;
        }
        return null;
    }
    public void refresh(TreeNode tn){
        if(tn == null)
            return;
        if(this.getCommonViewer() != null){
            try{
                this.getCommonViewer().refresh(tn);
            }
            catch(Exception e){
                //TODO fix unnecessary refresh
            }
        }
    }
    @Override
    protected Object getInitialInput() {
        if(super.getInitialInput()==null) {
            return ResourcesPlugin.getWorkspace().getRoot();
        }
        return super.getInitialInput();
    }
}
