package com.unitesk.requality.eclipse.ui.editors;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.UUID;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.views.contentoutline.IContentOutlinePage;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.NodeMapper;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.handlers.location.CommonRemoveLocation;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectTestPurposeDialog;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

public abstract class JSEditor extends EditorPart implements ISelectionProvider {

    public static final int EDIT_MODE_REQ_RO = 0x01;
    public static final int EDIT_MODE_TP_RO = 0x02;
    private Browser browser;
    protected File html;
    private TreeNode node;
    private TreeNode currentSelection;
    private TreeDB db;
    private UniEditorOutlinePage uniEditorOutlinePage;
    private int editMode = 0;

    private ITreeStateListener treeStateListener = new ITreeStateListener() {
        
        @Override
        public void open(TreeDB db) {
            // TODO Auto-generated method stub
            
        }
        
        @Override
        public void close(TreeDB db) {
            if(!JSEditor.this.db.equals(db)) {
                return;
            }
            closeEditor();
        }

        @Override
        public void delete(IProject proj) {
            IProject project = TreesTracker.getTracker().getProject(db);
            if (!project .equals(proj)) {
                return;
            }
            closeEditor();
        }

    };

    private ISelectionChangedListener outlineSelectionChangeListener = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            IStructuredSelection selection = (IStructuredSelection) event
                    .getSelection();
            if (selection.isEmpty()) {
                return;
            }
            setUniEditorFocus(((TreeNode) selection.getFirstElement())
                    .getUUId());
            currentSelection = (TreeNode) selection.getFirstElement();
            // synchronization with Requality Explorer
            IViewPart view = PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getActivePage()
                    .findView(RequalityCNF.ID);
            if (view instanceof CommonNavigator) {
                CommonNavigator navigator = (CommonNavigator) view;
                if (navigator.isLinkingEnabled()) {
                    navigator.setLinkingEnabled(true);
                }
            }
        }
    };

    public TreeNode getCurrentSelection() {
        if(currentSelection == null) {
            currentSelection = targetNode;
        }
        return currentSelection;
    }

    public String getSelectionText() {
        String res = (String) browser.evaluate("return getSelectionText();");
        if (res == null)
            return "";
        StringTokenizer st = new StringTokenizer(res, "\n ");
        res = "";
        while (st.hasMoreTokens()) {
            if (!res.isEmpty()) {
                res += " ";
            }
            res += st.nextToken();
        }
        return res;
    }
    
    public void setSelection(TreeNode node) {
        if(node == null){
            currentSelection = node;
            return;
        }
        setUniEditorFocus(node.getUUId());
        currentSelection = node;
        if (uniEditorOutlinePage != null
                && !uniEditorOutlinePage.getControl().isDisposed()) {
            uniEditorOutlinePage.setSelection(new StructuredSelection(node));
        }
    }

    public boolean findText(String aString, boolean aCaseSensitive,
            boolean aBackwards, boolean aWrapAround) {
        return (Boolean) browser.evaluate("return findText('" + aString
                + "'," + aCaseSensitive + "," + aBackwards + "," + aWrapAround
                + ");");
    }

    private Map<UUID, NodeDesc> changedNodes;
    private Map<UUID, Integer> changedNodesCount;
    ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            refresh(node.getParent());
            refresh(oldParent);
        }

        @Override
        public void deleted(NodeDesc nodeDesc) {
            if(!db.getStorage().canLoad(JSEditor.this.node.getUUId())) { // current node was deleted
                closeEditor();
                return;
            }
            if(db.getStorage().canLoad(nodeDesc.getParentUUId())) {
                refresh(db.getNode(nodeDesc.getParentUUId()));
            }
        }

        @Override
        public void changed(TreeNode node) {
            refresh(node);
            refresh(node.getParent());
        }

        @Override
        public void created(TreeNode node) {
            refresh(node.getParent());
        }
    };
    //Check if id is valid
    private static boolean reqIdValid(TreeNode req, TreeNode parent, String text){
        if(req == null && parent == null) return false;
        return TreeNode.isValidId(req, parent, text);
    }
    //Check if child can be added to parent
    private static boolean reqChildValid(TreeNode req, TreeNode parent, String name){
        if(req == null && parent == null) return false;
        return TreeNode.isValidFileName(name) && parent.getPossibleChildTypes().contains(req.getType());
    }
    private TreeNode targetNode;

    @Override
    public void doSave(IProgressMonitor monitor) {
    }
    @Override
    public void doSaveAs() {
    }
    private String tabLabel = "";
    /* (non-Javadoc)
     * setter for Editor's prefix in tab label 
     */
    protected void setTabLabel(String value){
        tabLabel = value;
    }
    /* (non-Javadoc)
     * Update a tab name
     */
    protected void updateTabLabel(){
        String partName;
        if (node instanceof Requirement) {
            partName = ((Requirement) node).getNameOrId();
        } else {
            partName = node.toString();
        }
        if(tabLabel.equals("")){
            setPartName(partName);
        }else{
            setPartName(tabLabel+" - "+partName);
        }
    }
    
    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        changedNodes = new HashMap<UUID, NodeDesc>();
        changedNodesCount = new HashMap<UUID, Integer>();
        TreeNode sel = null;
        try {
            node = ((NodeEditorInput) input).getStorage().getNode();
            db = node.getTreeDB();
            updateTabLabel();
            sel = ((NodeEditorInput) input).getSelectionStorage().getNode();
        } catch (CoreException e) {
            e.printStackTrace();
        }
        setSite(site);
        setInput(input);
        targetNode = sel;
        TreesTracker.getTracker().addListener(treeStateListener);
        getSite().setSelectionProvider(this);
        //Two handlers to deals with command's exceptions
        IHandlerService handlerService = (IHandlerService) getSite()
                .getService(IHandlerService.class);
        IHandler copyhandler = new AbstractHandler() {
            public Object execute(ExecutionEvent event)
                  throws ExecutionException {
            return null;
          }
        };
        IHandler pastehandler = new AbstractHandler() {
            public Object execute(ExecutionEvent event)
                    throws ExecutionException {
              return null;
            }
          };
        handlerService
            .activateHandler("org.eclipse.ui.edit.copy", copyhandler);
        handlerService
            .activateHandler("org.eclipse.ui.edit.paste", pastehandler);
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    protected String node2jsonString(TreeNode n) throws JSONException {
        String result;
        JSONObject json = NodeMapper.toJSON(n);
        json.put("id", n.getId());
        JSONArray jsonChildren = new JSONArray();
        TreeNode[] children = n.getSortedChildren();
        for (int i = 0; i < children.length; i++) {
            jsonChildren.put(children[i].getUUId().toString());
        }
        json.put("children", jsonChildren);

        JSONObject attr = json.getJSONObject("attributes");
        @SuppressWarnings("rawtypes")
        Iterator keys = attr.keys();
        while (keys.hasNext()) {
            String key = (String) keys.next();
            JSONObject a = attr.getJSONObject(key);
            if (n.getQualifiedId().equals(Requirement.getTypeRootQId())) {
                if (a.getString("key").equals(TreeNode.ATTR_TYPE)) {
                    a.putOpt("value", null);
                }
            }
            if (a.getString("key").equals(TreeNode.ATTR_DESCRIPTION)) {
                String descr = n.getDescription(true);
                descr = RequalityCLI.resolveRelativeImagesPath(descr, n);
                a.putOpt("value", descr);
            }
            if (n.getType().equals(TestPurpose.TYPE_NAME) && a.getString("key").equals(TestPurpose.ATTR_EXPECTED_RESULTS)) {
                a.putOpt("value", ((TestPurpose)n).getExpectedResults(true));
            }
        }
        result = json.toString(4);
        return result;
    }

    private void deleteLocations(Requirement req) {
        for (Location loc : req.getLocations()) {
            CommonRemoveLocation.RemoveLocation(loc, req, null, true);
            //TODO: save jsData for undo/redo
        }
        for (int i = 0; i < req.sizeChildren(); i++) {
            TreeNode node = req.getChild(i);
            if (node instanceof Requirement) {
                Requirement r = (Requirement) node;
                deleteLocations(r);
            }
        }
    }

    @Override
    public void createPartControl(Composite parent) {
        browser = new Browser(parent, RequalityPlugin.BROWSER_TYPE);
        browser.addMouseListener(new MouseListener() {
            @Override
            public void mouseUp(MouseEvent e) {
            }
            
            @Override
            public void mouseDown(MouseEvent e) {
                JSEditor.this.setFocus();
            }
            @Override
            public void mouseDoubleClick(MouseEvent e) {
            }
        });

        browser.addProgressListener(new ProgressListener() {
            
            @Override
            public void completed(ProgressEvent event) {
                setSelection(currentSelection);
            }
            
            @Override
            public void changed(ProgressEvent event) {
            }
        });

        try {
            browser.setUrl(html.toURI().toURL().toString());

            new BrowserFunction(browser, "getNodeByUUID") {
                @Override
                public Object function(Object[] arguments) {
                    UUID nodeUUID = UUID.fromString((String) arguments[0]);
                    TreeNode n = db.getNode(nodeUUID);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };

            new BrowserFunction(browser, "getNodeByQID") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode((String) arguments[0]);
                    String result;
                    try {
                        result = node2jsonString(n);
                    } catch (JSONException e) {
                        e.printStackTrace();
                        return null;
                    }
                    return result;
                }
            };
            /*
             * Request for the root element of the selected subtree Input
             * parameters: none Return value: String value containing uuid of
             * the subtree root node
             */
            new BrowserFunction(browser, "getStartNode") {
                @Override
                public Object function(Object[] arguments) {
                    return node.getUUId().toString();
                }
            };
            
            new BrowserFunction(browser, "getSelectedNode") {
                @Override
                public Object function(Object[] arguments) {
                    if(currentSelection == null)
                        return node.getUUId().toString();
                    return currentSelection.getUUId().toString();
                }
            };

            /*
             * Request to change the node ID Input parameters: uuid - (String)
             * uuid of the updated node new_id - (String) new ID to be set
             * Return value: Boolean value: true if operation was successful,
             * false otherwise
             */
            new BrowserFunction(browser, "changeNodeId") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String newId = (String) arguments[1];
                    if (newId.equals("")) {
                        refresh(n);
                    } else {
                        db.startTransaction("Change Node Id");
                        n.setId(newId);
                        db.commit();
                    }
                    return true;
                }
            };

            new BrowserFunction(browser, "changeNodeName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    if (!n.getType().equals(Requirement.TYPE_NAME)) {
                        return false;
                    }
                    String newName = (String) arguments[1];
                    db.startTransaction("Change Node Name");
                    ((Requirement) n).setName(newName);
                    n.saveAttributes();
                    db.commit();
                    return true;
                }
            };

            new BrowserFunction(browser, "getUFName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    return n.getUserFriendlyName();
                }
            };
            /*
             * Returns true if it is possible to edit this node
             */
            new BrowserFunction(browser, "canEditNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    return n.canBeDeleted();
                }
            };
            /*
             * Check the possibility of adding some node as
             *  child of another. Also can be used to check
             *  posibily of adding a child with some name
             * 
             */
            new BrowserFunction(browser, "checkName") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = null;
                    String newName = (String) arguments[1];
                    if(!((String)arguments[0]).equals("")){
                        n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                        if(n.getType().equals(Requirement.TYPE_NAME)){
                            Attribute a = n.getAttribute(Requirement.ATTR_NAME);
                            if(a != null && a.getValue() != null && a.getValue() != "")
                                return true;
                        }
                        if(newName.equals(""))
                            newName = n.getId();
                    }
                    TreeNode parent = null;
                    if(arguments.length > 2){
                        parent = db.getNode(UUID
                                .fromString((String) arguments[2]));
                    }
                    if((n == null) && (parent == null) || newName.equals(""))
                        return null;
                    if(n != null && n.getType().equals(Comment.TYPE_NAME))
                        return true;
                    return reqIdValid(n,parent,newName);
                }
            };
            /*
             * Check if it possible to add some node 
             * as child of another one
             * 
             */
            new BrowserFunction(browser, "checkChild") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = null;
                    String newName = (String) arguments[1];
                    if(!((String)arguments[0]).equals("")){
                        n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                        if(n.getType().equals(Requirement.TYPE_NAME)){
                            Attribute a = n.getAttribute(Requirement.ATTR_NAME);
                            if(a != null && a.getValue() != null && a.getValue() != "")
                                return true;
                        }
                        if(newName.equals(""))
                            newName = n.getId();
                    }
                    TreeNode parent = null;
                    if(arguments.length > 2){
                        parent = db.getNode(UUID
                                .fromString((String) arguments[2]));
                    }
                    if((n == null) && (parent == null) || newName.equals(""))
                        return null;
                    if(n != null && n.getType().equals(Comment.TYPE_NAME))
                        return true;
                    return reqChildValid(n,parent,newName);
                }
            };
            
            /*
             * Request to change the value of one of the node attributes Input
             * parameters: uuid - (String) uuid of the updated node attr_name -
             * (String) name of the changed attribute new_val - (String) new
             * attribute value to be set Return value: Boolean value: true if
             * operation was successful, false otherwise
             */
            new BrowserFunction(browser, "changeNodeAttr") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        UUID uuid = UUID.fromString((String) arguments[0]);
                        TreeNode n = db.getNode(uuid);
                        String key = (String) arguments[1];
                        String value = (String) arguments[2];
                        synchronized (JSEditor.this) {
                            if (n.getType().equals(Requirement.TYPE_NAME) && key.equals(Requirement.ATTR_NAME) && value.equals("")) {
                                refresh(n);
                            } else
                            if (n.getType().equals(Requirement.TYPE_NAME) && key.equals(Requirement.ATTR_NAME) && ((Requirement)n).getName() == null) {
                                if (value.equals("")) {
                                    refresh(n);
                                } else {
                                    db.startTransaction("Change Node Id");
                                    ((Requirement)n).setId(value);
                                    db.commit();
                                }
                            } else {
                                db.startTransaction("Change Node Attribute");
                                if (key.equals(TreeNode.ATTR_DESCRIPTION) || key.equals(TestPurpose.ATTR_EXPECTED_RESULTS)) {
                                    RequalityCLI.deleteOldImages(key, value, n);
                                    value = RequalityCLI.copyImages(value, n);
                                    value = value.replace("\"" + n.getResourcesPath(), "\"${node.resURL}");
                                }
                                Attribute attr = null;
                                if (!value.equals(""))
                                {
                                    attr = new Attribute(AttributeType.STRING, key, value);
                                    n.putAttribute(attr);
                                } else {
                                    n.removeAttribute(key);
                                }
                                if (!changedNodes.containsKey(uuid)) {
                                    changedNodes.put(uuid, n.toNodeDesc());
                                    changedNodesCount.put(uuid, 1);
                                } else {
                                    NodeDesc nodeDesc = changedNodes.get(uuid);
                                    if (attr != null) {
                                        nodeDesc.setAttribute(attr);
                                    } else {
                                        nodeDesc.removeAttribute(key);
                                    }
                                    changedNodesCount.put(uuid, changedNodesCount.get(uuid) + 1);
                                }
                                n.saveAttributes();
                                db.commit();
                            }
                        }
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                    return true;
                }
            };

            /*
             * Request to change the move a node into different subtree Input
             * parameters: uuid - (String) uuid of the moved node
             * new_parent_uuid - (String) uuid of the new parent node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "changeNodeParent") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[1]));
                    Index i = null;
                    if(n.getType().equals("TestStep") 
                            && (parent.getType().equals("TestStep") || parent.getType().equals("TestProcedure"))){
                        TreeNode node = null;
                        if(parent.getType().equals("TestStep")){
                            node = parent.getParent();
                        }else{
                            node = parent;
                            parent = null;
                        }
                        i = getNextTestStepIndex(node, parent);
                    }
                    if(!parent.getPossibleChildTypes().contains(n.getType()))
                        parent = parent.getParent();
                    if(!parent.canBeParentOf(n) || !n.canBeChildOf(parent))
                        return false;
                    if (!n.getParent().equals(parent) || (i != null)) {
                        if(n == null || parent == null || (!n.canBeChildOf(parent) || !parent.canBeParentOf(n)))
                            return false;
                        db.startTransaction("Move Node");
                        if(i != null){
                            n.setIndex(i);
                            n.saveAttributes();
                        }else{
                            if(n.getType().equals(Comment.TYPE_NAME)){
                                if (parent.findChild(n.getId()) != null) {
                                    n.setId(RequalityCLI.getNextId(parent, n.getType(), null));
                                }
                            }
                            n.setParent(parent);
                        }
                        db.commit();
                        setSelection(n);
                        refresh(parent);
                        return true;
                    }
                    return false;
                }
            };

            /*
             * Request to insert a new node Input parameters: uuid - (String)
             * uuid of the new node parent_uuid - (String) uuid of the parent
             * node Return value: Boolean value: true if operation was
             * successful, false otherwise
             */
            new BrowserFunction(browser, "addNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode parent = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    String id = (String) arguments[1];
                    String type = (String) arguments[2];
                    if(!parent.canBeParentOf(type))
                        return null;
                    if (id == null || id.length() == 0) {
                        if (type.equals(Comment.TYPE_NAME)) {
                            id = RequalityCLI.getNextId(parent, type, Comment.TYPE_NAME + " ");
                        } else {
                            id = RequalityCLI.getNextId(parent, type, null);
                        }
                    }
                    if (parent.findChild(id) == null) {
                        if(!parent.canBeParentOf(type))
                            return null;
                        db.startTransaction("Add Node");
                        TreeNode newNode = parent.createChild(id, type);
                        //TODO: remake to commands
                        if (type.equals(TestPurpose.TYPE_NAME)) {
                            newNode.putAttribute(new Attribute(AttributeType.STRING,
                                    TestPurpose.ATTR_AUTHOR, System.getProperty("user.name", "")));
                            newNode.putAttribute(new Attribute(AttributeType.STRING,
                                    TestPurpose.ATTR_STATUS, "in process"));
                            newNode.saveAttributes();
                        }
                        db.commit();
                        return newNode.getUUId().toString();
                    }
                    return null;
                }
            };

            /*
             * Request to remove a node with all its decendants Input
             * parameters: uuid - (String) uuid of the removed node Return
             * value: Boolean value: true if operation was successful, false
             * otherwise
             */
            new BrowserFunction(browser, "removeNode") {
                @Override
                public Object function(Object[] arguments) {
                    TreeNode n = db.getNode(UUID
                            .fromString((String) arguments[0]));
                    if(!n.canBeDeleted())
                        return false;
                    db.startTransaction("Remove Node");
                    if (n instanceof Requirement) {
                        deleteLocations((Requirement)n);
                    }
                    n.delete();
                    db.commit();
                    return true;
                }
            };
            new BrowserFunction(browser, "setRequalityFocus") {
                @Override
                public Object function(Object[] arguments) {
                    // workaround (ignore first unieditor selection change)
                    if(currentSelection == null) {
                        currentSelection = targetNode!=null?targetNode:db.getNode(UUID
                                .fromString((String) arguments[0]));
                    } else {
                        currentSelection = db.getNode(UUID
                                .fromString((String) arguments[0]));
                    }
                    if (uniEditorOutlinePage != null) {
                        uniEditorOutlinePage
                                .setSelection(new StructuredSelection(
                                        currentSelection));
                    }
                    // synchronization with Requality Explorer
                    IViewPart view = PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getActivePage()
                            .findView(RequalityCNF.ID);
                    if (view instanceof CommonNavigator) {
                        CommonNavigator navigator = (CommonNavigator) view;
                        if (navigator.isLinkingEnabled()) {
                            navigator.setLinkingEnabled(true);
                        }
                    }
                    fireSelectionChange();
                    return true;
                }
            };

            /**
             * returns absolute URL to project folder
             */
            new BrowserFunction(browser, "getBaseUrl") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        return RequalityPlugin.getTreesTracker().getProject(db)
                                .getLocation().toFile().toURI().toURL()
                                .toString();
                    } catch (MalformedURLException e) {
                        e.printStackTrace();
                    }
                    return null;
                }
            };

            new BrowserFunction(browser, "getReadOnlyMode") {
                @Override
                public Object function(Object[] arguments) {
                    return editMode;
                }
            };

            /**
             * uuid - src node uuid new_parent_uuid - dst node (new parent) uuid
             * deep - optional argument (default deep == true) copy subtree
             * returns absolute URL to project folder
             */
            new BrowserFunction(browser, "copyNode") {
                @Override
                public Object function(Object[] arguments) {
                    try {
                        TreeNode srcNode = db.getNode(UUID
                                .fromString((String) arguments[0]));
                        TreeNode newParent = db.getNode(UUID
                                .fromString((String) arguments[1]));
                        TreeNode selNode = null;
                        Index in = null;
                        if(srcNode.getType().equals("TestStep")){
                            selNode = newParent;
                        }
                        if(!newParent.getPossibleChildTypes().contains(srcNode.getType()))
                            newParent = newParent.getParent();
                        if(!srcNode.canBeCopiedTo(newParent) || !(newParent.canAddCopyOf(srcNode)))
                            return false;
                        Boolean deep = true;
                        if (arguments.length > 2) {
                            deep = (Boolean) arguments[2];
                        }
                        if(arguments.length > 3)
                            selNode = db.getNode(UUID
                                    .fromString((String) arguments[3]));
                        NodeDesc nd = srcNode.toNodeDesc(deep);
                        if (newParent.findChild(nd.getId()) != null) {
                            nd.setId(RequalityCLI.getNextId(newParent, srcNode.getType(), null));
                        }
                        db.startTransaction("Copy Node");
                        if(srcNode.getType().equals("TestStep"))
                            in = getNextTestStepIndex(newParent, selNode);
                        TreeNode newNode = nd.createNode(srcNode.getTreeDB(),
                                newParent.getUUId(), deep);
                        if(srcNode.getType().equals("TestStep")){
                            newNode.setIndex(in);
                            newNode.saveAttributes();
                        }
                        db.commit();
                        setSelection(newNode);
                        refresh(newParent);
                        // FIXME: DataBase affected???
                        copyNodeResources(srcNode, newNode);
                        return newNode.getUUId().toString();
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                    return null;
                }

                private void copyNodeResources(TreeNode srcNode,
                        TreeNode newNode) {
                    IFolder srcFolder = TreesTracker
                            .getResourceByNode(srcNode)
                            .getParent()
                            .getFolder(
                                    Path.fromOSString(srcNode.getId()
                                            + "_resources"));
                    if (!srcFolder.exists()) {
                        return;
                    }
                    IFolder destination = TreesTracker
                            .getResourceByNode(newNode)
                            .getParent()
                            .getFolder(
                                    Path.fromOSString(newNode.getId()
                                            + "_resources"));
                    try {
                        srcFolder.copy(destination.getFullPath(), false, null);
                    } catch (CoreException e) {
                        e.printStackTrace();
                    }

                }
            };
            
            new BrowserFunction(browser, "getLastChildUUID") {
                @Override
                public Object function( Object[] arguments ) {
                    TreeNode parent = db.getNode( UUID.fromString( (String)arguments[0] ) );
                    TreeNode[] children = parent.getSortedChildren();
                    if( children == null || children.length == 0 ) return null;
                    return children[children.length-1].getUUId().toString();
                }
            };

            // [0] - test procedure UUID
            // [1] - previous test step UUID
            new BrowserFunction( browser, "createTestStep" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    TreeNode parent = db.getNode( UUID.fromString((String) arguments[0]));

                    if( arguments.length < 2 || arguments[1] == null || ((String)arguments[1]).length() == 0 )
                    {
                        // no children
                        createTestStep( parent, Index.createOne() ); 
                        return null;
                    }
                    TreeNode node = db.getNode( UUID.fromString((String) arguments[1]));
                    createTestStep(parent, getNextTestStepIndex(parent, node));
                    return null;
                }
            };
            
            new BrowserFunction( browser, "addTestPurposeToTestStep" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    TreeNode node = db.getNode( UUID.fromString((String) arguments[0]));
                    SelectNodeDialog selDialog = new SelectTestPurposeDialog(
                            getSite().getShell(), db);
                    if (selDialog.open() == SelectNodeDialog.OK) {
                        TreeNode res = selDialog.getResult();
                        //TODO ss
                        node.getTreeDB().startTransaction("Set Test Purpose");
                        node.putAttribute(new Attribute(AttributeType.STRING,"_tslink",res.getUUId().toString()));
                        node.saveAttributes();
                        node.getTreeDB().commit();
                        refresh(node);
                        refresh(res);
                        return res.getUUId().toString();
                    }
                    return null;
                }
            };
            
            new BrowserFunction( browser, "openEditorForNode" )
            {
                @Override
                public Object function( Object[] arguments )
                {
                    final TreeNode nde = db.getNode( UUID.fromString((String) arguments[0]));
                    TreeNode node = nde;
                    final TreeNode sel = nde;
                            String tp = node.getType();
                            if(tp.equals(TestPurpose.TYPE_NAME)
                                    ||tp.equals(Comment.TYPE_NAME)
                                    ||tp.equals("TestStep")
                                    ||tp.equals(Location.TYPE_NAME))
                                node = node.getParent();
                            NodeEditorInput input = new NodeEditorInput(node, sel);
                            String editorid = UniEditor.ID;
                            try 
                            {
                                if(tp.equals(Document.TYPE_NAME))
                                    editorid = ReqMarker.ID;
                                /*final IEditorPart idp = */ IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), input, editorid, false);
//                                PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
//                                    public void run() {
//                                        //Taken from handler
//                                        try {
//                                            wait(500);
//                                        } catch (InterruptedException e) {
//                                            e.printStackTrace();
//                                        }
//                                        
//                                    }});                                
                                //if(idp instanceof JSEditor)
                                  //((JSEditor)idp).setSelection(sel);
                            } 
                            catch(PartInitException e) 
                            {
                                e.printStackTrace();
                            }
                    return null;
                }
            };
            
        } catch (IOException e) {
            e.printStackTrace();
        }
        db.addListener(treeChangeListener);
    }
    @Override
    public void setFocus() {
        new UndoRedoActionGroup(getSite(), PlatformUI.getWorkbench()
                .getOperationSupport().getUndoContext(), true)
                .fillActionBars(getEditorSite().getActionBars());
        browser.setFocus();
    }
    private Index getNextTestStepIndex(TreeNode parent,TreeNode node){
        Index ind = Index.createOne();
        TreeNode[] children = parent.getSortedChildren();
        if (node == null){
            if(children.length > 0)
                ind = children[0].getIndex().findMiddleIndex(Index.getZero());
            return ind;
        }
        boolean found = false;
        for( int i = 0; i < children.length; i++ )
        {
            TreeNode n1 = children[i];
            if( n1.getUUId().equals( node.getUUId() ) )
            {
                found = true;
                if( i < children.length - 1 )
                {
                    TreeNode n2 = children[i+1]; 
                    return n1.getIndex().findMiddleIndex( n2.getIndex() );
                } else {
                    return n1.getIndex().findGreaterIndex();
                }
            }
        }
        if(!found)
            if(children.length > 0)
                ind = children[0].getIndex().findMiddleIndex(Index.getZero());
        return ind;
    }
    public void dispose() {
        db.removeListener(treeChangeListener);
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    public JSEditor() {
        super();
        try {
            html = new File(Activator.getPath("jstools/unieditor.html"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    protected void refresh(final TreeNode node) {
        // Refresh nothing but root node subtree
        String rootQID = this.node.getQualifiedId();
        String nodeQID = node.getQualifiedId();
        
        if (!nodeQID.startsWith(rootQID + "/") && (!nodeQID.equals(rootQID))) {
            if(node.getType().equals("TestPurpose")&&(this.node.getType().equals("TestProcedure")))
                browser.execute("testStepUpdate('" + node.getUUId()
                        + "');");
            return;
        }
        if(rootQID.equals(nodeQID))
            updateTabLabel();
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (browser != null && !browser.isDisposed()) {
                    synchronized (JSEditor.this) {
                        if (changedNodes.containsKey(node.getUUId())) {
                            NodeDesc nd = changedNodes.get(node.getUUId());
                            if (nd.equals(node.toNodeDesc())) {
                                Integer count = changedNodesCount.get(node
                                        .getUUId());
                                if (count > 0) {
                                    changedNodesCount.put(node.getUUId(),
                                            changedNodesCount.get(node
                                                    .getUUId()) - 1);
                                    return;
                                }
                            }
                            changedNodes.remove(node.getUUId());
                            changedNodesCount.remove(node.getUUId());
                        }
                        browser.execute("onChangeNode('" + node.getUUId()
                                + "');");
                    }
                }
            }
        });
    }

    protected void setUniEditorFocus(final UUID uuId) {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (browser != null && !browser.isDisposed()) {
                    browser.execute("onChangeFocus('" + uuId + "');");
                }
            }
        });
    }

    @SuppressWarnings("rawtypes")
    @Override
    public Object getAdapter(Class required) {
        if (IContentOutlinePage.class.equals(required)) {
            if (uniEditorOutlinePage == null) {
                uniEditorOutlinePage = new UniEditorOutlinePage();
                uniEditorOutlinePage
                        .addSelectionChangedListener(outlineSelectionChangeListener);
                uniEditorOutlinePage.setInput(this.node);
            }
            return uniEditorOutlinePage;
        }
        if (required == IPropertySheetPage.class) {
            return new TabbedPropertySheetPage(new ITabbedPropertySheetPageContributor() {
                @Override
                public String getContributorId() {
                    return "com.unitesk.requality.views.RequalityPropertyContributor";
                }
            });
        }
        return super.getAdapter(required);
    }

    public Browser getBrowser() {
        return browser;
    }

    public void setEditMode(int editMode) {
        this.editMode = editMode;
    }

    private void closeEditor() {
        getSite().getShell().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                getEditorSite().getPage()
                        .closeEditor(JSEditor.this, false);
            }
        });
    }

    // //////////////////////////////////////////////////////////////////////////
    //
    // Selection provider
    //
    // //////////////////////////////////////////////////////////////////////////

    private List<ISelectionChangedListener> selectionChangeListeners = new ArrayList<ISelectionChangedListener>();

    @Override
    public void addSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangeListeners.add(listener);
    }

    @Override
    public void removeSelectionChangedListener(
            ISelectionChangedListener listener) {
        selectionChangeListeners.remove(listener);
    }

    @Override
    public ISelection getSelection() {
        if (currentSelection == null) {
            return StructuredSelection.EMPTY;
        }
        return new StructuredSelection(currentSelection);
    }

    @Override
    public void setSelection(ISelection selection) {
    }

    private void fireSelectionChange() {
        ISelection sel = this.getSelection();
        SelectionChangedEvent event = new SelectionChangedEvent(this, sel);
        for (ISelectionChangedListener listener : selectionChangeListeners) {
            listener.selectionChanged(event);
        }
    }

    private TreeNode createTestStep( TreeNode parent, Index index )
    {
        String type = "TestStep";
        String id = RequalityCLI.getNextId(parent, type, null);
        if (parent.findChild(id) == null) {
            if(!parent.canBeParentOf(type))
                return null;
            db.startTransaction("Add Test Step");
            TreeNode newNode = parent.createChild( id, type );
            newNode.setIndex( index );
            newNode.saveAttributes();
            db.commit();
            return newNode;
        }
        return null;
    }

}
