package com.unitesk.requality.eclipse.ui.editors;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.model.WorkbenchLabelProvider;
import org.eclipse.ui.operations.UndoRedoActionGroup;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.contentoutline.ContentOutlinePage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalitySorter;
import com.unitesk.requality.eclipse.views.RequalityContentProvider;

public class UniEditorOutlinePage extends ContentOutlinePage {
    private TreeNode input;
    @Override
    public void dispose() {
        input.getTreeDB().removeListener(treeChangeListener);
        super.dispose();
    }

    ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(final TreeNode node, final TreeNode oldParent) {
            UIJob job = new UIJob("update outline") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                    if(!UniEditorOutlinePage.this.getSite().getShell().isDisposed()){
                        UniEditorOutlinePage.this.refresh(oldParent);
                        UniEditorOutlinePage.this.refresh(node);
                        return Status.OK_STATUS;
                    }
                    return Status.CANCEL_STATUS;
                }
            };
            job.schedule();
        }

        @Override
        public void deleted(NodeDesc nodeDesc) {
        }

        @Override
        public void changed(final TreeNode node) {
            UIJob job = new UIJob("update outline") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                    if(!UniEditorOutlinePage.this.getSite().getShell().isDisposed()){
                        UniEditorOutlinePage.this.refresh(node);
                        return Status.OK_STATUS;
                    }
                    return Status.CANCEL_STATUS;
                }
            };
            job.schedule();
        }

        @Override
        public void created(TreeNode node) {
        }
    };

    public UniEditorOutlinePage() {
    }

    @Override
    public void createControl(Composite parent) {
        super.createControl(parent);
        getTreeViewer().setLabelProvider(
                WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider());
        getTreeViewer().setContentProvider(new RequalityContentProvider());
        getTreeViewer().setSorter(new RequalitySorter());
        if (input != null) {
            getTreeViewer().setInput(new Object[] { input });
        }
        getTreeViewer().expandAll();
        IWorkbenchPartSite site = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getActivePage().getActivePart()
                .getSite();
        IUndoContext context = PlatformUI.getWorkbench()
                .getOperationSupport().getUndoContext();
        new UndoRedoActionGroup(site, context, true)
                .fillActionBars(getSite().getActionBars());

    }

    public void setInput(TreeNode node) {
        if(input != null)
            input.getTreeDB().removeListener(treeChangeListener);
        input = node;
        if (getTreeViewer() != null) {
            getTreeViewer().setInput(node);
        }
        input.getTreeDB().addListener(treeChangeListener);
    }

    public synchronized void refresh(final TreeNode node){
        getSite().getShell().getDisplay().syncExec(new Runnable() {
            @Override
            public void run() {
                //if control is disposing then old code may
                //throw exception
                if ((UniEditorOutlinePage.this.getControl() != null) &&
                        !UniEditorOutlinePage.this.getControl().isDisposed()) {
                    getTreeViewer().refresh(node);
                }
            }
        });
    }
}
