/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.TestPurpose;

public class TestDescriptionPropertySection extends TreeNodePropertiesTab {

	private TestPurpose testPurpose;
	
	private Text description; // Test Case Description
	private Text results;     // Expected Results

	public TestDescriptionPropertySection() {
	}

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = 
        		aTabbedPropertySheetPage.getWidgetFactory();

        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        wf.createLabel(parent, "Test purpose description:");
        
        description = wf.createText(parent, "", SWT.MULTI | SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        description.setLayoutData(new GridData(GridData.FILL_BOTH));
        description.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                String oldDescr = testPurpose.getDescription();
                if (oldDescr == null) {
                    oldDescr = "";
                }
                String newDescr = "";
                newDescr = description.getText();
                if (oldDescr.equals(newDescr)) {
                    return;
                }
                testPurpose.getTreeDB().startTransaction("Update Test Purpose");
                if (!newDescr.equals("")) {
                	testPurpose.setDescription(newDescr);
                } else {
                	testPurpose.removeAttribute(TreeNode.ATTR_DESCRIPTION);
                }
                testPurpose.saveAttributes();
                testPurpose.getTreeDB().commit();
            }
        });

        wf.createLabel(parent, "Expected results:");

        results = wf.createText(parent, "", SWT.MULTI | SWT.BORDER | SWT.H_SCROLL
                | SWT.V_SCROLL);
        results.setLayoutData(new GridData(GridData.FILL_BOTH));
        results.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                String oldResults = testPurpose.getExpectedResults(false);
                if (oldResults == null) {
                	oldResults = "";
                }
                String newResults = "";
                newResults = results.getText();
                if (oldResults.equals(newResults)) {
                    return;
                }
                testPurpose.getTreeDB().startTransaction("Update Test Purpose");
                if (!newResults.equals("")) {
                	testPurpose.setExpectedResults(newResults);
                } else {
                	testPurpose.removeAttribute(TestPurpose.ATTR_EXPECTED_RESULTS);
                }
                testPurpose.saveAttributes();
                testPurpose.getTreeDB().commit();
            }
        });
        
    }
    
	@Override
	protected void setTreeNode(TreeNode node) {
        if (!(node instanceof TestPurpose)
        		|| description == null || description.isDisposed()) {
            return;
        }
		testPurpose = (TestPurpose) node;
		// getDescription() returns null instead of ""
		// if corresponding attribute isn't presented in node
		description.setText(
				testPurpose.getDescription() != null ? testPurpose.getDescription() : ""
				);
		results.setText(testPurpose.getExpectedResults(false));
    }
}
