/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;

public class TestMainPropertySection extends TreeNodePropertiesTab {
    private TestPurpose testPurpose;

    private Text testPurposeID; // Test Case Number
    private Combo status; // Status
    private Text author; // Author
    private AttributesPanel attribPane; // Attributes

    private ControlDecoration testPurposeIDDecorator;

    public TestMainPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = getWidgetFactory();

        parent.addFocusListener(new FocusListener() {
            @Override
            public void focusLost(FocusEvent e) {
            }

            @Override
            public void focusGained(FocusEvent e) {
            }
        });
        parent.setLayout(new GridLayout(2, false));
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));

        wf.createLabel(parent, "Id:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        testPurposeID = wf.createText(parent, "", SWT.BORDER);
        testPurposeID.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        testPurposeIDDecorator = createDecorator(testPurposeID);
        testPurposeIDDecorator.hide();
        testPurposeID.addModifyListener(new ModifyListener() {
            @Override
            public void modifyText(ModifyEvent e) {
                if (isReqIdOk(testPurposeID.getText())) {
                    testPurposeIDDecorator.hide();
                } else {
                    if (testPurposeID.getText().trim().equals("")) {
                        testPurposeIDDecorator
                                .setDescriptionText("Test Purpose Id should not be empty");
                    } else {
                        testPurposeIDDecorator
                                .setDescriptionText("Test Purpose Id is not correct or Test Purpose with the same Id already exists");
                    }
                    testPurposeIDDecorator.show();
                }
            }
        });
        testPurposeID.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                String newId = testPurposeID.getText().trim();
                if (testPurpose.getId().equals(newId)) {
                    return;
                }
                if (!testPurpose.getQualifiedId().equals(
                        Requirement.getTypeRootQId())) {
                    if (isReqIdOk(newId)) {
                        testPurpose.getTreeDB().startTransaction(
                                "Update Test Purpose");
                        testPurpose.setId(newId);
                        testPurpose.getTreeDB().commit();
                    } else {
                        testPurposeID.setText(testPurpose.getId());
                    }
                }
            }
        });

        wf.createLabel(parent, "Status:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        status = new Combo(parent, SWT.BORDER | SWT.READ_ONLY);
        status.setItems(new String[] { "in process", "complete", "verified" });
        status.addSelectionListener(new SelectionListener() {
            public void widgetSelected(SelectionEvent e) {
                String oldStatus = (String) testPurpose
                        .getAttributeValue(TestPurpose.ATTR_STATUS);
                if (oldStatus == null) {
                    oldStatus = "";
                }
                String newStatus = "";
                newStatus = status.getText();
                if (oldStatus.equals(newStatus)) {
                    return;
                }
                testPurpose.getTreeDB().startTransaction("Update Test Purpose");
                testPurpose.putAttribute(new Attribute(AttributeType.STRING,
                        TestPurpose.ATTR_STATUS, newStatus));
                testPurpose.saveAttributes();
                testPurpose.getTreeDB().commit();
            }

            public void widgetDefaultSelected(SelectionEvent e) {
            }
        });

        wf.createLabel(parent, "Author:").setLayoutData(
                new GridData(GridData.HORIZONTAL_ALIGN_END));

        author = wf.createText(parent, "", SWT.BORDER);
        author.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        author.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
                String oldAuthor = (String) testPurpose
                        .getAttributeValue(TestPurpose.ATTR_AUTHOR);
                if (oldAuthor == null) {
                    oldAuthor = "";
                }
                String newAuthor = "";
                newAuthor = author.getText();
                if (oldAuthor.equals(newAuthor)) {
                    return;
                }
                testPurpose.getTreeDB().startTransaction("Update Test Purpose");
                if (!newAuthor.equals("")) {
                    testPurpose.putAttribute(new Attribute(
                            AttributeType.STRING, TestPurpose.ATTR_AUTHOR,
                            newAuthor));
                } else {
                    testPurpose.removeAttribute(TestPurpose.ATTR_AUTHOR);
                }
                testPurpose.saveAttributes();
                testPurpose.getTreeDB().commit();
            }
        });

        attribPane = new AttributesPanel(parent, wf);
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalSpan = 2;
        attribPane.getControl().setLayoutData(gridData);
    }

    private ControlDecoration createDecorator(Text text) {
        ControlDecoration controlDecoration = new ControlDecoration(text,
                SWT.LEFT | SWT.TOP);
        FieldDecoration fieldDecoration = FieldDecorationRegistry.getDefault()
                .getFieldDecoration(FieldDecorationRegistry.DEC_ERROR);
        controlDecoration.setImage(fieldDecoration.getImage());
        return controlDecoration;
    }

    protected boolean isReqIdOk(String text) {
        return TreeNode.isValidId(this.testPurpose, text);
    }

    @Override
    protected void setTreeNode(TreeNode node) {
        if (!(node instanceof TestPurpose) || testPurposeID == null
                || testPurposeID.isDisposed()) {
            return;
        }
        testPurpose = (TestPurpose) node;
        testPurposeID.setText(testPurpose.getId());
        status.select(status.indexOf((String) testPurpose.getAttributeValue(
                TestPurpose.ATTR_STATUS, "")));
        author.setText((String) testPurpose.getAttributeValue(
                TestPurpose.ATTR_AUTHOR, ""));
        attribPane.setNode(testPurpose);
    }
}
