/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package com.unitesk.requality.eclipse.wizards;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.internal.resources.Project;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementTreeSelectionDialog;
import org.eclipse.ui.dialogs.FileSystemElement;
import org.eclipse.ui.dialogs.WizardResourceImportPage;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.internal.ide.dialogs.IElementFilter;
import org.eclipse.ui.model.AdaptableList;
import org.eclipse.ui.model.WorkbenchContentProvider;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.wizards.datatransfer.FileSystemStructureProvider;
import org.eclipse.ui.wizards.datatransfer.IImportStructureProvider;

import com.unitesk.requality.core.ITreeNodeFilter;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.documents.OmnivorousImporter;
import com.unitesk.requality.eclipse.core.ResourceStorage;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.eclipse.views.RequalityContentProvider;
import com.unitesk.requality.eclipse.views.documents.DocLabelProvider;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

@SuppressWarnings("restriction")
public class DocumentImportWizardPage extends WizardResourceImportPage
        implements Listener {

    private Combo sourceNameField;
    private boolean entryChanged;
    private Button sourceBrowseButton;
    private Button selectTypesButton;
    private Button selectAllButton;
    private Button deselectAllButton;
    private TreeNode importTarget;
    private Button renameExistingCheckbox;
    private Button createFoldersCheckbox;
    private IStatus importStatus;
    
    public DocumentImportWizardPage(IWorkbench workbench,
            IStructuredSelection selection) {
        this("RequalityImportWizardPage1", selection);
        if (selection != null
                && selection.getFirstElement() instanceof DocFolder) {
            DocFolder folder = (DocFolder) selection.getFirstElement();
            IProject prj = TreesTracker.getTracker().getProject(
                    folder.getTreeDB());
            super.setContainerFieldValue(prj.getName()
                    + folder.getQualifiedId());
            importTarget = folder;
        } else if (selection != null
                && selection.getFirstElement() instanceof Project) {
            TreeDB db = TreesTracker.getTracker().getTree(
                    (Project) selection.getFirstElement());
            importTarget = db.getNode(Document.getTypeRootQId());
        } else
            importTarget = null;
        selectRequalityTypes();
    }

    @SuppressWarnings("unchecked")
    private void selectRequalityTypes() {
        super.selectedTypes.add("xhtml");
        super.selectedTypes.add("html");
        super.selectedTypes.add("htm");
        super.selectedTypes.add("doc");
        super.selectedTypes.add("docx");
        super.selectedTypes.add("pdf");
    }

    public void handleEvent(Event event) {
        if (event.widget == sourceBrowseButton)
            handleSourceBrowseButtonPressed();
        super.handleEvent(event);
    }

    public boolean finish() {
        IUndoContext context = PlatformUI.getWorkbench().getOperationSupport().getUndoContext();
        PlatformUI.getWorkbench().getOperationSupport().getOperationHistory().dispose(context, true, true, true);
        if (!ensureSourceIsValid())
            return false;

        saveWidgetValues();

        @SuppressWarnings("unchecked")
        Iterator<FileSystemElement> resourcesEnum = getSelectedResources()
                .iterator();
        List<Object> fileSystemObjects = new ArrayList<Object>();
        while (resourcesEnum.hasNext()) {
            FileSystemElement e = resourcesEnum.next();
            if (!e.isDirectory())
                fileSystemObjects.add(e.getFileSystemObject());
        }

        if (!createFoldersCheckbox.getSelection()) {
            if (fileSystemObjects.size() > 0)
                return importFiles(fileSystemObjects, importTarget);
        } else {
            // TODO: undo document import
            // importTarget.getTreeDB().startTransaction();
            Map<TreeNode, List<Object>> fileMap = createFolders(importTarget,
                    fileSystemObjects);
            // importTarget.getTreeDB().commit();
            Iterator<TreeNode> parents = fileMap.keySet().iterator();
            while (parents.hasNext()) {
                TreeNode importDest = parents.next();
                if (!importFiles(fileMap.get(importDest), importDest))
                    return false;
            }
            return true;
        }
        MessageDialog.openInformation(getContainer().getShell(),
                "Document import", "No files were selected"); // TEXT7 (TEXT8)
        return false;
    }

    private boolean importFiles(List<Object> files, TreeNode parentFolder) {
        if (!importResources(files, parentFolder)) {
            if (!importStatus.isOK() && importStatus.getException() != null) {
                ErrorDialog.openError(getContainer().getShell(),
                        "Import failed",
                        "Fatal error occured during the importing process",
                        importStatus);
                importStatus = null;
                return false;
            } else if (!importStatus.isOK()) {
                ErrorDialog.openError(getContainer().getShell(),
                        "Import problem",
                        "Error occured during the importing process",
                        importStatus);
                importStatus = null;
                return true;
            }
        }
        return true;
    }

    //start transaction and commit, please!
    private Map<TreeNode, List<Object>> createFolders(TreeNode importTarget,
            List<Object> files) {
        IPath sourcePath = new Path(sourceNameField.getText());
        Map<TreeNode, List<Object>> resMapping = new HashMap<TreeNode, List<Object>>();
        for (Object o : files) {
            File f = (File) o;
            IPath filePath = new Path(f.getPath());
            IPath relativePath = filePath.removeFirstSegments(
                    sourcePath.matchingFirstSegments(filePath))
                    .removeLastSegments(1);
            TreeNode parent = createFolder(importTarget, relativePath);
            if (parent == null)
                parent = importTarget;
            if (resMapping.get(parent) == null)
                resMapping.put(parent, new ArrayList<Object>());
            resMapping.get(parent).add(o);
        }
        return resMapping;
    }

    //start transaction and commit, please!
    private TreeNode createFolder(TreeNode parent, IPath folderPath) {
        if (parent instanceof DocFolder && !folderPath.isEmpty()) {
            String folderName = folderPath.segment(0);
            TreeNode folder = parent.findChild(folderName);
            if (folder == null)
                folder = parent.createChild(folderName, parent.getType());
            if (folderPath.segmentCount() > 1)
                return createFolder(folder, folderPath.removeFirstSegments(1));
            return folder;
        }
        return null;
    }

    @Override
    protected void createOptionsGroupButtons(Group optionsGroup) {
        renameExistingCheckbox = new Button(optionsGroup, SWT.CHECK);
        renameExistingCheckbox.setFont(optionsGroup.getFont());
        renameExistingCheckbox.setText("Rename existing documents"); // TEXT1
        renameExistingCheckbox.addSelectionListener(new SelectionListener() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                updatePageCompletion();
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                updatePageCompletion();
            }

        });

        createFoldersCheckbox = new Button(optionsGroup, SWT.CHECK);
        createFoldersCheckbox.setFont(optionsGroup.getFont());
        createFoldersCheckbox.setText("Create folders"); // TEXT2
        createFoldersCheckbox.addSelectionListener(new SelectionListener() {

            @Override
            public void widgetSelected(SelectionEvent e) {
                updatePageCompletion();
            }

            @Override
            public void widgetDefaultSelected(SelectionEvent e) {
                updatePageCompletion();
            }

        });
    }

    @Override
    protected boolean validateOptionsGroup() {
        if (renameExistingCheckbox == null)
            return false;
        else if (!renameExistingCheckbox.getSelection()
                && resourcesExists(super.getSelectedResources())) {
            super.setErrorMessage("Document with the same name already exists"); // TEXT13
            return false;
        } else {
            super.setErrorMessage(null);
            return true;
        }
    }

    @Override
    protected boolean validateSourceGroup() {
        /*
         * if(new File(this.sourceNameField.getText()).exists()) {
         * super.selectionGroup.setFocus();
         * if(super.selectionGroup.getAllListItems().size()==1) {
         * super.selectionGroup.setAllSelections(true); } return
         * super.validateSourceGroup(); } return false;
         */
        return new File(this.sourceNameField.getText()).exists()
                && super.validateSourceGroup();
    }

    protected boolean resourcesExists(List<?> resources) {
        for (Object resource : resources) {
            if (resourceExists(resource) > 0)
                return true;
        }
        return false;
    }

    protected int resourceExists(Object resource) {
        return resourceExists(resource, null,
                createFoldersCheckbox.getSelection());
    }

    private int resourceExists(Object resource, TreeNode folder,
            boolean createFolders) {
        File fileToImport = null;
        if (resource instanceof MinimizedFileSystemElement)
            fileToImport = (File) ((MinimizedFileSystemElement) resource)
                    .getFileSystemObject();
        else if (resource instanceof File)
            fileToImport = (File) resource;
        else
            // wtf just happened?
            return 0;
        String filename = fileToImport.getName();
        filename = filename.substring(0, filename.lastIndexOf("."));
        if (folder == null && importTarget == null)
            return 0;
        else if (folder == null)
            folder = importTarget;
        if (createFolders) {
            String parentFolder = fileToImport.getParentFile().getName();
            DeepFirstTreeWalker walker = new DeepFirstTreeWalker(folder);
            boolean sameFolderAlreadyExists = false;
            for (TreeNode descendant : walker) {
                if (descendant.getId().equals(parentFolder)||descendant.equals(folder)) {
                    sameFolderAlreadyExists = true;
                    break;
                }
            }

            if (!sameFolderAlreadyExists)
                return 0;
        }
        TreeNode[] children = folder.getSortedChildren(Document.TYPE_NAME);
        int counter = 0;
        for (TreeNode doc : children) {
            if (doc instanceof Document) {
                String docName = doc.getId();
                docName = docName.substring(0, docName.lastIndexOf("."));
                if (docName.equals(filename))
                    counter++;
                if (counter != 0 && docName.startsWith(filename + "_"))
                    counter++;
            }
        }
        return counter;
    }

    protected boolean importResources(final List<Object> fileSystemObjects,
            final TreeNode importDestination) {
        try {
            Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                    .getShell();
            ProgressMonitorDialog dialog = new ProgressMonitorDialog(shell);
            dialog.run(true, false, new IRunnableWithProgress() {
                public void run(IProgressMonitor monitor) {
                    //error flag for more than 2 docs
                    Boolean error=false;
                    TreeDB db = importDestination.getTreeDB();
                    monitor.beginTask(
                            "Importing files to " + importDestination.getId(),
                            fileSystemObjects.size()); // TEXT10
                    try {
                        // TODO: Transaction start
                        // db.startTransaction();
                        for (Object file : fileSystemObjects) {
                            File sourceFile = (File) file;
                            importFile(sourceFile, importDestination);
                            monitor.worked(1);
                        }
                        // ODO:  Transaction end
                        // db.commit();
                        
                    } catch (Exception e) {
                        error = true;
                        e.printStackTrace();
                        updateStatus(IStatus.ERROR, "Fatal error", e);
                    } finally {
                        IProject proj = TreesTracker.getTracker()
                                .getProject(db);
                        
                        try {
                            proj.refreshLocal(IResource.DEPTH_INFINITE, null);
                        } catch (CoreException e) {
                            e.printStackTrace();
                        }
                        monitor.done();
                    }
                    //if there is no error and we have exactly 1 loaded document
                    if((!error && (fileSystemObjects.size() == 1)) && (lastImportedDoc != null)) {
                        PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
                            public void run() {
                                //Taken from handler
                                NodeEditorInput input = new NodeEditorInput(lastImportedDoc);
                                try 
                                {
                                    IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), input, ReqMarker.ID); 
                                } 
                                catch(PartInitException e) 
                                {
                                    e.printStackTrace();
                                }
                                IViewPart view = PlatformUI.getWorkbench()
                                        .getActiveWorkbenchWindow().getActivePage()
                                        .findView(RequalityCNF.ID);
                                if (view instanceof CommonNavigator) {
                                    CommonNavigator navigator = (CommonNavigator) view;
                                    navigator.selectReveal(new StructuredSelection(lastImportedDoc));
                                }
                            }
                            });
                    }
                }
                private Document lastImportedDoc;
                private void importFile(File sourceFile,
                        TreeNode importDestination) throws Exception {
                    String filename = sourceFile.getName();
                    filename = filename.substring(0,
                            filename.lastIndexOf("."))
                            + ".xhtml";
                    int counter;
                    if ((counter = resourceExists(sourceFile,
                            importDestination, false)) > 0) {
                        filename = filename.substring(0,
                                filename.lastIndexOf("."))
                                + "_" + counter + ".xhtml";
                    }
                    Document doc = (Document) importDestination
                            .createChild(filename, Document.TYPE_NAME);
                    doc.setResourceName(filename);
                    doc.saveAttributes();
                    
                    ResourceStorage rs = TreesTracker.getResourceStorage(doc
                            .getTreeDB());
                    IFolder resFolder = rs.getResourceFolder(doc
                            .getUUId());
                    if (!resFolder.exists()) {
                        resFolder.create(false, true, null);
                    }

                    File outFile = new File(resFolder.getLocation()
                            .toOSString(), filename);

                    if (!outFile.exists() || outFile.equals(sourceFile)) {
                        OmnivorousImporter importer = new OmnivorousImporter();

                        String docId = doc.getQualifiedId();
                        importer.importDocument(docId, sourceFile,
                                outFile, importDestination.getTreeDB(), (Requirement) importDestination.getTreeDB()
                                        .getNode(Requirement
                                                .getTypeRootQId()));
                        lastImportedDoc=doc;
                    }
                }
            });
        } catch (InvocationTargetException e) {
            e.printStackTrace();
            updateStatus(IStatus.ERROR, "Fatal error", e);
        } catch (InterruptedException e) {
            e.printStackTrace();
            updateStatus(IStatus.ERROR, "Fatal error", e);
        }
        if (importStatus != null)
            return false;
        return true;
    }

    private void updateStatus(int severity, String message, Throwable exception) {
        if (importStatus == null || !importStatus.isMultiStatus()) {
            importStatus = new MultiStatus("Requality", 0, message, null);
        }
        ((MultiStatus) importStatus).add(new Status(severity, "Requality",
                message, exception));
    }

    protected boolean ensureSourceIsValid() {
        if (new File(getSourceDirectoryName()).isDirectory())
            return true;

        displayErrorDialog("Source path doesn't exist"); // TEXT9
        sourceNameField.setFocus();
        return false;
    }

    private String getSourceDirectoryName() {
        return getSourceDirectoryName(this.sourceNameField.getText());
    }

    protected void handleSourceBrowseButtonPressed() {

        String currentSource = this.sourceNameField.getText();
        DirectoryDialog dialog = new DirectoryDialog(
                sourceNameField.getShell(), SWT.SAVE);
        dialog.setText("Document import"); // TEXT2
        dialog.setMessage("Select source folder"); // TEXT3
        dialog.setFilterPath(getSourceDirectoryName(currentSource));

        String selectedDirectory = dialog.open();
        if (selectedDirectory != null) {
            if ((getSourceDirectory(selectedDirectory) == null)
                    || selectedDirectory.equals(currentSource))
                return;
            setErrorMessage(null);
            setSourceName(selectedDirectory);
        }
    }

    @Override
    protected void handleContainerBrowseButtonPressed() {
        displayDestinationSelectionDialog();
    }

    private class DestinationSelectionDialog extends ElementTreeSelectionDialog {

        public DestinationSelectionDialog(Shell parent,
                ILabelProvider labelProvider,
                ITreeContentProvider contentProvider) {
            super(parent, labelProvider, contentProvider);
        }

        @Override
        protected Label createMessageArea(Composite parent) {
            Button createFolder = super.createButton(parent, -1,
                    "Create folder", false); // TEXT16
            GridData data = new GridData();
            data.horizontalAlignment = SWT.END;
            data.horizontalSpan = 2;
            createFolder.setLayoutData(data);
            createFolder.addSelectionListener(new SelectionListener() {

                @Override
                public void widgetSelected(SelectionEvent e) {
                    TreeViewer v = getTreeViewer();
                    IStructuredSelection selection = (IStructuredSelection) v
                            .getSelection();
                    if (selection != null
                            && selection.getFirstElement() instanceof DocFolder) {
                        TreeNode parentFolder = (TreeNode) selection
                                .getFirstElement();
                        InputDialog folderName = new InputDialog(getShell(),
                                "Create folder", "Folder name:", null, null); // TEXT17,
                        // TEXT18
                        folderName.open();
                        // TODO: import undo
                        // parentFolder.getTreeDB().startTransaction();
                        TreeNode newFolder = createFolder(parentFolder,
                                new Path(folderName.getValue()));
                        // parentFolder.getTreeDB().commit();
                        v.setSelection(new StructuredSelection(newFolder));
                    }
                    v.refresh();
                }

                @Override
                public void widgetDefaultSelected(SelectionEvent e) {
                }

            });
            return null;
        }
    }

    private void displayDestinationSelectionDialog() {
        RequalityContentProvider provider = new RequalityContentProvider();
        provider.addFilter(new ITreeNodeFilter() {
            @Override
            public boolean select(TreeNode node) {
                return node instanceof DocFolder;
            }
        });
        DestinationSelectionDialog dialog = new DestinationSelectionDialog(
                this.getShell(), new DocLabelProvider(), provider);
        dialog.setAllowMultiple(false);
        if (importTarget != null) {
            dialog.setInitialSelection(this.importTarget);
            dialog.setInput(importTarget.getTreeDB().getRootNode());
        } else {
            dialog.setInput(ResourcesPlugin.getWorkspace().getRoot());
        }
        dialog.setTitle("Import into Folder"); // TEXT
        dialog.open();
        Object[] result = dialog.getResult();
        if (result != null && result.length > 0
                && result[0] instanceof DocFolder) {
            DocFolder target = (DocFolder) result[0];
            IProject prj = TreesTracker.getTracker().getProject(
                    target.getTreeDB());
            super.setContainerFieldValue(prj.getName()
                    + target.getQualifiedId());
            this.importTarget = target;
        }
    }

    private String getSourceDirectoryName(String sourceName) {
        IPath result = new Path(sourceName.trim());

        if (result.getDevice() != null && result.segmentCount() == 0)
            result = result.addTrailingSeparator();
        else
            result = result.removeTrailingSeparator();

        return result.toOSString();
    }

    private File getSourceDirectory(String selectedDirectory) {
        File sourceDirectory = new File(
                getSourceDirectoryName(selectedDirectory));
        if (!sourceDirectory.exists() || !sourceDirectory.isDirectory()) {
            return null;
        }
        return sourceDirectory;
    }

    private void setSourceName(String selectedDirectory) {
        if (selectedDirectory.length() > 0) {

            String[] currentItems = this.sourceNameField.getItems();
            int selectionIndex = -1;
            for (int i = 0; i < currentItems.length; i++) {
                if (currentItems[i].equals(selectedDirectory))
                    selectionIndex = i;
            }
            if (selectionIndex < 0) {
                int oldLength = currentItems.length;
                String[] newItems = new String[oldLength + 1];
                System.arraycopy(currentItems, 0, newItems, 0, oldLength);
                newItems[oldLength] = selectedDirectory;
                this.sourceNameField.setItems(newItems);
                selectionIndex = oldLength;
            }
            this.sourceNameField.select(selectionIndex);

            resetSelection();
        }
    }

    private void resetSelection() {
        MinimizedFileSystemElement currentRoot = getFileSystemTree();
        this.selectionGroup.setRoot(currentRoot);
    }

    private MinimizedFileSystemElement getFileSystemTree() {
        File sourceDirectory = getSourceDirectory();
        if (sourceDirectory == null)
            return null;

        return selectFiles(sourceDirectory,
                FileSystemStructureProvider.INSTANCE);
    }

    private MinimizedFileSystemElement selectFiles(final File sourceDirectory,
            final FileSystemStructureProvider instance) {
        final MinimizedFileSystemElement[] results = new MinimizedFileSystemElement[1];

        BusyIndicator.showWhile(getShell().getDisplay(), new Runnable() {
            public void run() {
                results[0] = createRootElement(sourceDirectory, instance);
            }
        });

        return results[0];
    }

    protected MinimizedFileSystemElement createRootElement(
            File sourceDirectory, FileSystemStructureProvider instance) {
        boolean isContainer = instance.isFolder(sourceDirectory);
        String elementLabel = instance.getLabel(sourceDirectory);

        MinimizedFileSystemElement dummyParent = new MinimizedFileSystemElement(
                "", null, true);
        dummyParent.setPopulated();
        MinimizedFileSystemElement result = new MinimizedFileSystemElement(
                elementLabel, dummyParent, isContainer);
        result.setFileSystemObject(sourceDirectory);
        result.getFiles(instance);

        return dummyParent;
    }

    private File getSourceDirectory() {
        return getSourceDirectory(this.sourceNameField.getText());
    }

    protected DocumentImportWizardPage(String name,
            IStructuredSelection selection) {
        super(name, selection);
    }

    @Override
    protected void createSourceGroup(Composite parent) {
        createSourceDirectoryGroup(parent);
        super.createFileSelectionGroup(parent);
        createButtonsGroup(parent);
    }

    private void createButtonsGroup(Composite parent) {
        Composite buttonComposite = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 3;
        layout.makeColumnsEqualWidth = true;
        buttonComposite.setLayout(layout);
        buttonComposite.setFont(parent.getFont());
        GridData buttonData = new GridData(GridData.VERTICAL_ALIGN_FILL
                | GridData.HORIZONTAL_ALIGN_FILL);
        buttonData.horizontalSpan = 2;
        buttonComposite.setLayoutData(buttonData);

        selectTypesButton = createButton(buttonComposite,
                IDialogConstants.SELECT_TYPES_ID, "Select types", false); // TEXT4

        SelectionListener listener = new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                handleTypesEditButtonPressed();
            }
        };
        selectTypesButton.addSelectionListener(listener);
        setButtonLayoutData(selectTypesButton);

        selectAllButton = createButton(buttonComposite,
                IDialogConstants.SELECT_ALL_ID, "Select all", false); // TEXT5

        listener = new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                setAllSelections(true);
                updateWidgetEnablements();
            }
        };
        selectAllButton.addSelectionListener(listener);
        setButtonLayoutData(selectAllButton);

        deselectAllButton = createButton(buttonComposite,
                IDialogConstants.DESELECT_ALL_ID, "Deselect all", false); // TEXT6

        listener = new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                setAllSelections(false);
                updateWidgetEnablements();
            }
        };
        deselectAllButton.addSelectionListener(listener);
        setButtonLayoutData(deselectAllButton);
    }

    private Button createButton(Composite parent, int id, String label,
            boolean defaultButton) {
        ((GridLayout) parent.getLayout()).numColumns++;

        Button button = new Button(parent, SWT.PUSH);
        button.setFont(parent.getFont());

        GridData buttonData = new GridData(GridData.FILL_HORIZONTAL);
        button.setLayoutData(buttonData);

        button.setData(new Integer(id));
        button.setText(label);

        if (defaultButton) {
            Shell shell = parent.getShell();
            if (shell != null) {
                shell.setDefaultButton(button);
            }
            button.setFocus();
        }
        return button;
    }

    private void createSourceDirectoryGroup(Composite parent) {
        Composite sourceContainerGroup = new Composite(parent, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 3;
        sourceContainerGroup.setLayout(layout);
        sourceContainerGroup.setFont(parent.getFont());
        sourceContainerGroup.setLayoutData(new GridData(
                GridData.HORIZONTAL_ALIGN_FILL | GridData.GRAB_HORIZONTAL));

        Label groupLabel = new Label(sourceContainerGroup, SWT.NONE);
        groupLabel.setText("From directory:"); // TEXT1
        groupLabel.setFont(parent.getFont());

        sourceNameField = new Combo(sourceContainerGroup, SWT.BORDER);
        GridData data = new GridData(GridData.HORIZONTAL_ALIGN_FILL
                | GridData.GRAB_HORIZONTAL);
        data.widthHint = SIZING_TEXT_FIELD_WIDTH;
        sourceNameField.setLayoutData(data);
        sourceNameField.setFont(parent.getFont());

        sourceNameField.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                updateFromSourceField();
            }
        });
        sourceNameField.addKeyListener(new KeyListener() {

            private String sourceValue;

            public void keyPressed(KeyEvent e) {
                // entryChanged = true;
                sourceValue = sourceNameField.getText();
                setPageComplete(false);
            }

            public void keyReleased(KeyEvent e) {
                if (!sourceValue.equals(sourceNameField.getText()))
                    entryChanged = true;
                if (e.keyCode == '\r') {
                    File sourcepath = new File(sourceNameField.getText());
                    if (sourcepath.exists() && sourcepath.isDirectory()) {
                        updateFromSourceField();
                    }/*
                      * else if(sourcepath.exists() && sourcepath.isFile()) {
                      * selectSingleFile(sourceNameField.getText());
                      * 
                      * }
                      */
                }
            }
        });
        sourceNameField.addFocusListener(new FocusListener() {
            public void focusGained(FocusEvent e) {
            }

            public void focusLost(FocusEvent e) {
                if (entryChanged) {
                    entryChanged = false;
                    updateFromSourceField();
                }
                determinePageCompletion();
            }
        });

        sourceBrowseButton = new Button(sourceContainerGroup, SWT.PUSH);
        sourceBrowseButton.setText("Browse...");
        sourceBrowseButton.addListener(SWT.Selection, this);
        sourceBrowseButton.setLayoutData(new GridData(
                GridData.HORIZONTAL_ALIGN_FILL));
        sourceBrowseButton.setFont(parent.getFont());
        setButtonLayoutData(sourceBrowseButton);
    }

    @SuppressWarnings("unchecked")
    /*
     * private void selectSingleFile(String filepath){ String parentFolder = new
     * File(filepath).getParent(); String filename = new
     * File(filepath).getName(); String extension = new
     * File(filepath).getName().substring(filename.indexOf('.')+1);
     * if(!extension.isEmpty() && !this.selectedTypes.contains(extension))
     * this.selectedTypes.add(extension); sourceNameField.setText(parentFolder);
     * updateFromSourceField(); Map<Object, Object> itemMap = new
     * HashMap<Object, Object>(); for(Object o :
     * super.selectionGroup.getAllListItems()) { if(o instanceof
     * MinimizedFileSystemElement) { MinimizedFileSystemElement element =
     * (MinimizedFileSystemElement) o; if(!element.isDirectory()) { File file =
     * (File) element.getFileSystemObject();
     * if(file.getPath().equals(filepath)){ List<MinimizedFileSystemElement>
     * selected = new ArrayList<MinimizedFileSystemElement>();
     * selected.add(element); itemMap.put(element.getParent(), selected); } } }
     * } super.selectionGroup.updateSelections(itemMap); }
     */
    @Override
    protected void setupSelectionsBasedOnSelectedTypes() {
        ProgressMonitorDialog dialog = new ProgressMonitorDialog(getContainer()
                .getShell());
        final Map<FileSystemElement, List<?>> selectionMap = new Hashtable<FileSystemElement, List<?>>();

        final IElementFilter filter = new IElementFilter() {

            @SuppressWarnings("rawtypes")
            public void filterElements(Collection files,
                    IProgressMonitor monitor) throws InterruptedException {
                if (files == null) {
                    throw new InterruptedException();
                }
                Iterator<?> filesList = files.iterator();
                while (filesList.hasNext()) {
                    if (monitor.isCanceled())
                        throw new InterruptedException();
                    checkFile(filesList.next());
                }
            }

            public void filterElements(Object[] files, IProgressMonitor monitor)
                    throws InterruptedException {
                if (files == null) {
                    throw new InterruptedException();
                }
                for (int i = 0; i < files.length; i++) {
                    if (monitor.isCanceled())
                        throw new InterruptedException();
                    checkFile(files[i]);
                }
            }

            private void checkFile(Object fileElement) {
                MinimizedFileSystemElement file = (MinimizedFileSystemElement) fileElement;
                if (isExportableExtension(file.getFileNameExtension())) {
                    List<FileSystemElement> elements = new ArrayList<FileSystemElement>();
                    FileSystemElement parent = file.getParent();
                    if (selectionMap.containsKey(parent))
                        elements = (List<FileSystemElement>) selectionMap
                                .get(parent);
                    elements.add(file);
                    selectionMap.put(parent, elements);
                }
            }

        };

        IRunnableWithProgress runnable = new IRunnableWithProgress() {
            public void run(final IProgressMonitor monitor)
                    throws InterruptedException {
                monitor.beginTask("Selectioning monitoring processing",
                        IProgressMonitor.UNKNOWN);
                getSelectedResources(filter, monitor);
            }
        };

        try {
            dialog.run(true, true, runnable);
        } catch (InvocationTargetException exception) {
            return;
        } catch (InterruptedException exception) {
            return;
        }
        getShell().update();
        if (selectionMap != null) {
            updateSelections(selectionMap);
        }
    }

    @SuppressWarnings("unchecked")
    protected boolean isExportableExtension(String extension) {
        if (selectedTypes == null) // ie.- all extensions are acceptable
            return true;

        Iterator<String> itr = selectedTypes.iterator();
        while (itr.hasNext()) {
            if (extension.equalsIgnoreCase((String) itr.next()))
                return true;
        }

        return false;
    }

    protected void updateFromSourceField() {
        setSourceName(sourceNameField.getText());
        super.updateWidgetEnablements();
    }

    @Override
    protected ITreeContentProvider getFileProvider() {
        return new WorkbenchContentProvider() {
            public Object[] getChildren(Object o) {
                if (o instanceof MinimizedFileSystemElement) {
                    MinimizedFileSystemElement element = (MinimizedFileSystemElement) o;
                    List<Object> exportableFiles = new ArrayList<Object>();
                    for (Object file : element.getFiles(
                            FileSystemStructureProvider.INSTANCE).getChildren(
                            element))
                        if (isExportableExtension(((MinimizedFileSystemElement) file)
                                .getFileNameExtension()))
                            exportableFiles.add(file);
                    return exportableFiles.toArray();
                }
                return new Object[0];
            }
        };
    }

    @Override
    protected ITreeContentProvider getFolderProvider() {
        return new WorkbenchContentProvider() {
            public Object[] getChildren(Object o) {
                if (o instanceof MinimizedFileSystemElement) {
                    MinimizedFileSystemElement element = (MinimizedFileSystemElement) o;
                    return element.getFolders(
                            FileSystemStructureProvider.INSTANCE).getChildren(
                            element);
                }
                return new Object[0];
            }

            public boolean hasChildren(Object o) {
                if (o instanceof MinimizedFileSystemElement) {
                    MinimizedFileSystemElement element = (MinimizedFileSystemElement) o;
                    if (element.getFiles().size() != 0)
                        return getChildren(element).length > 0;

                    return true;
                }
                return false;
            }
        };
    }

    protected class MinimizedFileSystemElement extends FileSystemElement {
        private boolean populated = false;

        public MinimizedFileSystemElement(String name,
                FileSystemElement parent, boolean isDirectory) {
            super(name, parent, isDirectory);
        }

        public AdaptableList getFiles(IImportStructureProvider provider) {
            if (!populated) {
                populate(provider);
            }
            return super.getFiles();
        }

        public AdaptableList getFolders(IImportStructureProvider provider) {
            if (!populated) {
                populate(provider);
            }
            return super.getFolders();
        }

        boolean isPopulated() {
            return this.populated;
        }

        boolean notPopulated() {
            return !this.populated;
        }

        private void populate(IImportStructureProvider provider) {

            Object fileSystemObject = getFileSystemObject();

            List<?> children = provider.getChildren(fileSystemObject);
            if (children == null) {
                children = new ArrayList<Object>(1);
            }
            Iterator<?> childrenEnum = children.iterator();
            while (childrenEnum.hasNext()) {
                Object child = childrenEnum.next();

                String elementLabel = provider.getLabel(child);
                MinimizedFileSystemElement result = new MinimizedFileSystemElement(
                        elementLabel, this, provider.isFolder(child));
                result.setFileSystemObject(child);
            }
            setPopulated();
        }

        public void setPopulated() {
            this.populated = true;
        }
    }
}