package com.unitesk.requality.merge;

import java.util.ArrayList;
import java.util.List;

public class Node {

    private org.w3c.dom.Node domNode;
    private Context ctx;
    private Node parent;
    private boolean isLeaf;
    private List<Node> childs;

    private Node() {
	this.isLeaf = true;
	childs = new ArrayList<Node>();
    }

    public Node(org.w3c.dom.Node domNode) {
	this();
	this.domNode = domNode;
	parent = null;
	ctx = new Context(this);
    }

    public Node(Node parentNode, org.w3c.dom.Node domNode) {
	this();
	this.parent = parentNode;
	this.domNode = domNode;
	this.ctx = new Context(this, parent.getContext());
	parentNode.isLeaf = false;
	parentNode.childs.add(this);
    }

    public org.w3c.dom.Node getDomNode() {
	return domNode;
    }

    public boolean isLeaf() {
	return isLeaf;
    }

    public List<Node> getChilds() {
	return childs;
    }

    public boolean isRoot() {
	if (parent == null)
	    return true;
	else
	    return false;
    }

    public boolean isEqual(Node node) {
	org.w3c.dom.Node otherDomNode = node.getDomNode();
	if (domNode.getNodeType() != otherDomNode.getNodeType())
	    return false;
	if (domNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE
		&& !domNode.getNodeValue().trim()
			.equals(otherDomNode.getNodeValue().trim()))
	    return false;
	if (!domNode.getNodeName().equals(otherDomNode.getNodeName()))
	    return false;
	return true;
    }

    public Context getContext() {
	return ctx;
    }

    public Node getParent() {
	return parent;
    }

    public String toString() {
	String str = "";
	if (parent != null)
	    str = parent + "/";
	else
	    str = str + "/";
	if (domNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE) {
	    if (domNode.getNodeValue().trim().isEmpty())
		str = str + "Whitespace" + "[" + ctx.getIndex() + "]";
	    else
		str = str + domNode.getNodeValue().trim();
	} else {
	    str = str + domNode.getNodeName() + "[" + ctx.getIndex() + "]";
	}
	return str;
    }

    public boolean isRelated(Node x) {
	if (isParent(x) || x.isParent(this))
	    return true;
	if (this.isSibling(x))
	    return true;
	return false;
    }

    public boolean isParent(Node x) {
	if (this.isRoot() && !x.isRoot() || !this.isRoot() && !x.isRoot()) {
	    if (x.getParent().isEqual(this))
		return true;
	    else
		return false;
	} else
	    return false;
    }

    public boolean isSibling(Node x) {
	if (this.parent == null || x.parent == null)
	    return false;
	if (this.getParent().isEqual(x.getParent()))
	    return true;
	else
	    return false;
    }

    public boolean equals(Node x) {
	return this.ctx.equals(x.ctx);
    }

    public boolean isText() {
	if (this.domNode.getNodeType() == org.w3c.dom.Node.TEXT_NODE)
	    return true;
	else
	    return false;
    }

    public String getText() {
	String text = "";
	if(this.isText())
	    return this.domNode.getNodeValue();
	else if(this.isLeaf)
	    return "";
	else
	    for(Node child : this.childs)
		text = text + child.getText();
	return text;
    }
    
    public boolean isSame(Node other) {
	if (this.domNode.isSameNode(other.domNode))
	    return true;
	else
	    return false;
    }

    public Node getPreviousSibling() {
	if (parent != null) {
	    List<Node> siblings = parent.childs;
	    int thisIndex = siblings.indexOf(this);
	    if (thisIndex != 0)
		return siblings.get(thisIndex - 1);
	}
	return null;
    }

    public Node getNextSibling() {
	if (parent != null) {
	    List<Node> siblings = parent.childs;
	    int thisIndex = siblings.indexOf(this);
	    if (thisIndex != siblings.size() - 1)
		return siblings.get(thisIndex + 1);
	}
	return null;
    }
    
    public List<Node> getDescendants() {
	List<Node> descendants = new ArrayList<Node>();
	descendants.add(this);
	for (int i = 0; i < descendants.size(); i++) {
	    Node currentNode = descendants.get(i);
	    if (currentNode.childs.isEmpty())
		continue;
	    for (Node child : currentNode.childs)
		descendants.add(child);
	}
	return descendants;
    }
}
