/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.nodetypes;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;

public class ReportData extends TreeNode {

    public static final String TYPE_NAME = "ReportData";
    public static final String ATTR_ROOT_REQUIREMENT = "_root_requiremnt_qid";
    public static final String ATTR_DATE = "date";
    public static final String ATTR_TEMPLATE = "_template_id";
    public ReportData() {
        super(null, null);
    }

    public ReportData(TreeDB db, UUID uuid) {
        super(db, uuid);
    }

    public String getStartRequirementQId() {
        return (String) getAttributeValue(ATTR_ROOT_REQUIREMENT, Requirement.getTypeRootQId());
    }

    public Map<String, Object> getData() {
        Map<String, Object> result = new HashMap<String, Object>();
        for (String key : getAttributeKeys()) {
            if (!key.startsWith("_")) {
                result.put(key, getAttributeValue(key));
            }
        }
        return result;
    }

    public String getSaveDir() {
        return getTreeDB().getStorage().getNodeResourcePath(getUUId());

    }

    public String getTemplateId() {
        return (String) getAttributeValue(ATTR_TEMPLATE);
    }
    private static List<String> possparents=new ArrayList<String>();
    private static List<String> posschildren=new ArrayList<String>();
    static{
        possparents.add(ReportFolder.TYPE_NAME);
    }
    @Override
    protected List<String> possibleParents() {
        return possparents;
    }
    @Override
    protected List<String> possibleChildren() {
        return posschildren;
    }
}
