package com.unitesk.requality.nodetypes;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;

public class TestPurpose extends TreeNode {

    public static final String TYPE_NAME = "TestPurpose";
    public static final String ATTR_EXPECTED_RESULTS = "_expectedResults";
    public static final String ATTR_STATUS = "_status";
    public static final String ATTR_AUTHOR = "_author";
    public static final String ATTR_NAME = "_name";
    public static final String ATTR_TEST_METHOD = "_testMethod";
    public static final String ATTR_TEST_LOCATION = "_testLocation";

    // Constructor for eclipse instancer
    public TestPurpose() {
        super(null, null);
    }
    
    private void fixAttributes() {
    	String oldResults = (String) getAttributeValue("expectedResults","");
    	if(oldResults == null || oldResults.isEmpty()) return;
    	removeAttribute("expectedResults");
    	setExpectedResults(getAttributeValue(ATTR_EXPECTED_RESULTS,"") + oldResults); 	
    }
    
    public TestPurpose(TreeDB db, UUID id) {
        super(db, id);
    }

    public String getTargetRequirementQId() {
        if(getParent()==null) return null;
        return getParent().getQualifiedId();
    }

    public void setTargetRequirementQId(String targetReqQId) {
        if(! targetReqQId.equals(getParent().getQualifiedId())) {
            setParent(getTreeDB().getNode(targetReqQId));
        }
    }

    public String getExpectedResults(boolean process) {
        fixAttributes();
        String result = (String) getAttributeValue(ATTR_EXPECTED_RESULTS,"");
        if (process) {
            result = processResourcesPath(result);
        }
        return result;
    }

    public void setExpectedResults(String expectedResults) {
        putAttribute(new Attribute(AttributeType.STRING,
                ATTR_EXPECTED_RESULTS, expectedResults));
    }

    public String getDescription(String def) {
        String result = getDescription();
        if(result==null) {
            return def;
        }
        return result;
    }
    
    @Override
    public String toString() {
        return getId();
    }
    
    private static List<String> possparents=new ArrayList<String>();
    private static List<String> posschildren=new ArrayList<String>();
    static{
        possparents.add(Requirement.TYPE_NAME);
        posschildren.add(Comment.TYPE_NAME);
    }
    @Override
    protected List<String> possibleParents(){
        return possparents;
    }
    
    @Override
    public boolean canBeChildOf(TreeNode parent) {
        if(parent.getType().equals(Requirement.TYPE_NAME))
            return ((Requirement)parent).canBeParentOf(this);
        return super.canBeChildOf(parent);
    }

    @Override
    protected List<String> possibleChildren(){
        return posschildren;
    }

    public boolean isCopyable(){
        return true;
    }

    @Override
    public String getUserFriendlyName() {
        if (getId().equals("Requirements"))
            return "";
        return super.getUserFriendlyName();
    }
    
}
