/*
 * Copyright 2009-2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URI;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.ReportGenerator;
import com.unitesk.requality.report.Template;

//
public class RequalityCLI {

    public final static String cmdName = "requality";
    private static Map<String, String> usages = new HashMap<String, String>();

    static {

        usages.put(
                "report",
                "Usage: "
                        + cmdName
                        + " report "
                        + "<path to template> <path to catalogue> <save location> [<root qid>]");
        usages.put(
                "convert",
                "Usage: "
                        + cmdName
                        + " convert "
                        + "<path to database> <path to document> [<document folder qid>] [<group requirement qid>]");
    }

    public static void usage() {
        log("Usage: " + cmdName + " " + "<help|convert|report> [params]");
    }

    public static void usageHelp(String param) {
        StringBuilder result = new StringBuilder();

        if (usages.containsKey(param)) {
            result.append(usages.get(param));
        } else {
            usage();
        }

        log(result.toString());
    }

    private static void log(String string) {
        System.out.println(string);
    }

    public static void main(String[] args) throws Exception {
        if (args.length == 0) {
            usage();
            return;
        }
        System.out
                .println("Welcome to Requality console mode! Process started...");

        if ("help".equals(args[0])) {
            String subj = args.length > 1 ? args[1] : "";
            usageHelp(subj);
            return;
        }
        // [<document folder qid>] [<group requirement qid>]
        if ("convert".equals(args[0])) {
            if (args.length > 2 && args.length < 6) {
                String pathToDB = args[1];
                String pathToDoc = args[2];
                String[] qids = new String[args.length - 3];
                for (int i = 3; i < args.length; i++) {
                    qids[i - 3] = args[i];
                }

                File inFile = new File(pathToDoc);
                TreeDB db = TreeDB.load(new FSStorage(pathToDB));
                db.addNodeType(Requirement.TYPE_NAME, Requirement.class);
                db.addNodeType(Location.TYPE_NAME, Location.class);
                db.addNodeType(DocFolder.TYPE_NAME, DocFolder.class);
                db.addNodeType(Document.TYPE_NAME, Document.class);
                Requirement groupNode = (Requirement) db.getNode(Requirement
                        .getTypeRootQId());
                DocFolder folderNode = (DocFolder) db.getNode(Document
                        .getTypeRootQId());

                db.startTransaction("convert");
                Document doc = (Document) folderNode.createChild(
                        inFile.getName(), Document.TYPE_NAME);
                db.commit();
                Converter c = new Converter();
                try {
                    Writer out = new FileWriter(doc.getHTMLPath());
                    Map<String, String[]> result = c.convert(new FileReader(
                            inFile), out);
                    c.store(result, doc.getQualifiedId(), db, groupNode);
                    System.out.println("OK");
                } catch (IOException e) {
                    System.out.println("ERROR");
                    e.printStackTrace();
                }
                return;
            }
            usageHelp("convert");
            return;
        }

        if ("report".equals(args[0])) {
            if (!(args.length >= 4 && args.length <= 5)) {
                usageHelp("report");
                return;
            }
            String reportArgs[] = Arrays.copyOfRange(args, 1, args.length);
            TreeDB db = TreeDB.load(new FSStorage(reportArgs[1]));
            db.addNodeType(Requirement.TYPE_NAME, Requirement.class);
            db.addNodeType(Location.TYPE_NAME, Location.class);
            db.addNodeType(DocFolder.TYPE_NAME, DocFolder.class);
            db.addNodeType(Document.TYPE_NAME, Document.class);

            String templatePath = reportArgs[0];
            Template tpl = new Template(templatePath);
            String rootQId = Requirement.getTypeRootQId();
            String saveTo = reportArgs[2];
            if (reportArgs.length > 3) {
                rootQId = reportArgs[3];
            }
            // TODO: inital data
            TreeDB instTree = TreeDB.load(new TreeInstancer(db, new HashMap<String, Object>()));
            
            instTree.addNodeType(Requirement.TYPE_NAME, Requirement.class);
            instTree.addNodeType(Location.TYPE_NAME, Location.class);
            instTree.addNodeType(DocFolder.TYPE_NAME, DocFolder.class);
            instTree.addNodeType(Document.TYPE_NAME, Document.class);

            Requirement rootReq = (Requirement) instTree.getNode(rootQId);
            // TODO: add user data
            ReportGenerator.generateReport(rootReq, tpl, saveTo,
                    Collections.<String, Object> emptyMap());
            return;
        }
        // if ("docclean".equals(args[0])) {
        // if (args.length == 2) {
        // docclean(args[1]);
        // } else {
        // System.out.println("neep path to database");
        // }
        // return;
        // }

        usage();
    }

    // private static void docclean(String path) {
    // TreeDB db = new TreeDBIO(path);
    // Map<String, Set<UUID>> docToUUIDs = new HashMap<String, Set<UUID>>();
    // for (TreeDBNode r : new DeepFirstTreeWalker(db.getRootNode(
    // Requirement.type, false))) {
    // Requirement req = (Requirement) r;
    // for (Location loc : req.getLocations()) {
    // String docId = loc.getDocId().replace('\\', '/');
    // if (docToUUIDs.get(docId) == null) {
    //
    // docToUUIDs.put(docId, new HashSet<UUID>());
    // }
    // docToUUIDs.get(docId).add(loc.getId());
    // }
    // }
    // for (String docId : docToUUIDs.keySet()) {
    // Set<UUID> locRemove = new HashSet<UUID>();
    // Document doc = db.getDocument(docId);
    // System.out.println("processing document: " + docId + " locations ("
    // + docToUUIDs.get(docId).size() + "/" + doc.sizeLocations()
    // + ")");
    //
    // for (UUID locId : doc.getLocations()) {
    // locRemove.add(locId);
    // }
    // locRemove.removeAll(docToUUIDs.get(docId));
    // for (UUID locId : locRemove) {
    // System.out.println("Remove location " + locId.toString()
    // + " from " + docId);
    // doc.removeLocation(locId);
    // }
    // }
    // }

    public static void copyfile(String srFile, String dtFile)
            throws IOException {
        File sourceFile = new File(srFile);
        File destFile = new File(dtFile);
        if (!sourceFile.exists()) {
            throw new IOException(sourceFile+" not exists");
        }
        if (!destFile.exists()) {
            destFile.createNewFile();
        }
        FileChannel source = null;
        FileChannel destination = null;
        try {
            source = new FileInputStream(sourceFile).getChannel();
            destination = new FileOutputStream(destFile).getChannel();
            destination.transferFrom(source, 0, source.size());
        } finally {
            if (source != null) {
                source.close();
            }
            if (destination != null) {
                destination.close();
            }
        }
    }

    public static String htmlEscape(String str) {
        return str.replace("&", "&amp;").replace("<", "&lt;")
                .replace(">", "&gt;");
    }

    private static String getNumberStringFromId(String in) {
        Pattern pattern = Pattern.compile("(.*\\D|)(\\d*)$");
        Matcher m = pattern.matcher(in);
        if (m.matches()) {
            return m.group(m.groupCount());
        } else {
            return null;
        }
    }

    public static String getNextId(TreeNode parent, String type, String defaultPrefix) {
        String newId = "00";
        String fullNewId = "00";
        boolean found = false;

        for (int i = 0; i < parent.sizeChildren(); i++) {
            if (parent.getChild(i).getType().equals(type)) {
                String number = getNumberStringFromId(parent.getChild(i).getId());
                if (number == null) {
                    continue;
                }
                try {
                    if (Integer.parseInt(number) >= Integer.parseInt(newId)) {
                        fullNewId = parent.getChild(i).getId();
                        newId = number;
                        found = true;
                    }
                } catch (NumberFormatException e) {
                }
            }
        }

        if (!found && defaultPrefix != null) {
            fullNewId = defaultPrefix + fullNewId;
        }

        do {
            fullNewId = incLastNumberInString(fullNewId);
        } while (parent.findChild(fullNewId) != null);
        return fullNewId;
    }

    // Utilites
    public static String incLastNumberInString(String str) {
        String regex = "\\d*";
        String tmp = str;

        while (!tmp.matches(regex))
            tmp = tmp.substring(1);
        String fixture = str.substring(0, str.length() - tmp.length());
        String tail = String.valueOf(Integer.parseInt(tmp) + 1);

        if (tail.length() < tmp.length()) {
            tail = makeZeroString(tmp.length() - tail.length()) + tail;
        }
        return fixture + tail;
    }

    private static String makeZeroString(int zeroCount) {
        StringBuilder sb = new StringBuilder(zeroCount);
        for (int i = 0; i < zeroCount; i++)
            sb.append("0");
        return sb.toString();
    }

    private static boolean isWindowsLocalPath(String path) {
        if (path.length() >= 2
                && (path.charAt(0) >= 'a' && path.charAt(0) <= 'z' || path
                        .charAt(0) >= 'A' && path.charAt(0) <= 'Z')
                && (path.charAt(1) == ':' || path.charAt(1) == '|'))
            return true;
        return false;
    }

    private final static Pattern patternImages = Pattern.compile("<img\\s+[^>]*src=\"([^\\\"]+)\"\\s*[^>]*>");

    public static String copyImages(String descr, TreeNode n) {
        Matcher matcher = patternImages.matcher(descr);
        while (matcher.find()) {
            String src = matcher.group(1);
            String pathToImage = src;
            pathToImage = pathToImage.replaceAll("\\\\", "/");
            // DELETE "file:/" PREFIXES
            // remote paths cases
            if (pathToImage.startsWith("file://")) {
                // Windows' shares
                if (pathToImage.startsWith("file://///")) {
                    pathToImage = pathToImage.substring(8);
                } else
                // "file://C:/Folder/..."
                if (isWindowsLocalPath(pathToImage.substring(7)) ||
                // "file:///C:/Folder/...", "file:///home/..." - remove
                // "file://"
                        pathToImage.startsWith("file:///")) {
                    pathToImage = pathToImage.substring(7);
                }
                // else "file://compname/share", prefix will be deleted below
            }
            // local paths cases - "file:/C:/Folder/...", "file:/home/..." -
            // remove "file:"
            if (pathToImage.startsWith("file:")) {
                pathToImage = pathToImage.substring(5);
            }
            // delete excess "/" on Windows
            if (pathToImage.startsWith("/")
                    && isWindowsLocalPath(pathToImage.substring(1))) {
                pathToImage = pathToImage.substring(1);
            }
            try {
                // search source file
                File srcFile = new File(pathToImage);
                if (srcFile.isAbsolute()) {
                    if (!srcFile.exists()) { // in URI
                        String absPath = srcFile.getAbsolutePath();
                        absPath = absPath.replaceAll("\\\\", "/");
                        if (isWindowsLocalPath(absPath)) {
                            absPath = "/" + absPath;
                        }
                        if (absPath.startsWith("//")) { // Windows' shares
                            absPath = "///" + absPath;
                        }
                        srcFile = new File(URI.create("file:" + absPath));
                    }
                } else {
                    // suppose, file in URI format
                    String rootPath = n.getTreeDB().getRootNode().getResourcesPath();
                    if (rootPath.endsWith("_resources")) { // hack
                        rootPath = rootPath.substring(0, rootPath.length() - 10) + "root";
                    }
                    try {
                        srcFile = new File(URI.create(rootPath + '/' + pathToImage));
                    } catch (IllegalArgumentException e) {
                        // no, file was not URI
                        File rootFile = new File(URI.create(rootPath));
                        // pathToImage = pathToImage.replaceAll("/", File.separator);
                        srcFile = new File(rootFile + File.separator + pathToImage);
                    }
                }
                if (!srcFile.exists()) {
                    continue;
                }
                // now srcFile is absolute not URI path
                // short file name in URI (with %20 as ' ' etc.)
                String shortDestFileName = (new File(srcFile.toURI().toURL().toString())).getName();
                int nSuffix = 0;
                //divide FileName into File Name and extension
                String ext = "";
                int dot = shortDestFileName.lastIndexOf('.');
                if (dot > -1) {
                    ext = shortDestFileName.substring(dot);
                    shortDestFileName = shortDestFileName.substring(0, dot);
                }
                // check destination file
                File dstFile = new File(URI.create(n.getResourcesPath() + '/' + shortDestFileName + ext));
                if (srcFile.getAbsolutePath().equals(dstFile.getAbsolutePath())) {
                    continue;
                }
                while (dstFile.exists()) {
                    nSuffix++;
                    dstFile = new File(URI.create(n.getResourcesPath() + '/' + shortDestFileName + nSuffix + ext));
                }
                // copy
                try {
                    FileInputStream in = new FileInputStream(srcFile);
                    n.setResourceContent(dstFile.getName(), in);
                    in.close();
                    descr = descr.replace(src, dstFile.toURI().toURL().toString());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }
        return descr;
    }

    public static String resolveRelativeImagesPath(String descr, TreeNode n) {
        if (!(n instanceof Location)) {
            return descr;
        }
        n = n.getParent();
        if (!(n instanceof Document)) {
            return descr;
        }
        Matcher matcher = patternImages.matcher(descr);
        while (matcher.find()) {
            String src = matcher.group(1);
            String pathToImage = src;
            pathToImage = pathToImage.replaceAll("\\\\", "/");
            pathToImage = n.getResourcesPath() + "/" + pathToImage;
            descr = descr.replace(src, pathToImage);
        }
        return descr;
    }

    public static void deleteOldImages(String attrName, String descr, TreeNode n) {
        Attribute attr = n.getAttribute(attrName);
        if (attr == null || attr.getType() != AttributeType.STRING) {
            return;
        }
        //get old images list
        String oldDescr = (String) attr.getValue();
        Matcher matcher = patternImages.matcher(oldDescr);
        List<String> oldImages = new ArrayList<String>();
        while (matcher.find()) {
            String src = matcher.group(1);
            src = src.replace("${node.resURL}", n.getResourcesPath());
            oldImages.add(src);
        }
        //get new images list
        matcher = patternImages.matcher(descr);
        List<String> newImages = new ArrayList<String>();
        while (matcher.find()) {
            newImages.add(matcher.group(1));
        }
        //remove duplicate items
        for (String newImg : newImages) {
            oldImages.remove(newImg);
        }
        //rest items - elements to delete
        for (String oldImg : oldImages) {
            oldImg = oldImg.replaceAll("\\\\", "/");
            if (oldImg.contains("/")) {
                oldImg = oldImg.substring(oldImg.lastIndexOf('/') + 1);
                n.deleteResource(oldImg);
            }
        }
    }

}
